
import sys
import os
import numpy as np
from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine
from sqlalchemy import schema, types
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import backref

Base = declarative_base()

class RunMetadata(Base):
    __tablename__ = 'RunMetadata'

    run_id = Column(Integer, primary_key=True)
    run_name = Column(String(250), nullable=False, index=True)
    watershed_id = Column(Integer, ForeignKey('Watershed.watershed_id'), nullable=True)
    pixel = Column(Integer, nullable=True)
    description = Column(String(250), nullable=True)
    smrf_version = Column(String(250), nullable=True)
    awsm_version = Column(String(250), nullable=True)
    snowav_version = Column(String(250), nullable=True)
    data_type = Column(String(250), nullable=True)
    data_location = Column(String(1200), nullable=True)
    file_type = Column(String(250), nullable=True)
    config_file = Column(String(250), nullable=True)
    proc_time = Column(types.DateTime(), nullable=True)

    # This puts Watershed fields available through Basin at Basin.watershed
    run_metadata = relationship('Watershed',
                               backref=backref('RunMetadata',lazy='dynamic'))

class Watershed(Base):
    __tablename__ = 'Watershed'

    watershed_id = Column(Integer, primary_key=True)
    watershed_name = Column(String(250), nullable=False, index=True)
    shapefile = Column(types.LargeBinary, nullable=True)


class Basin(Base):
    __tablename__ = 'Basin'

    watershed_id = Column(Integer,
                          ForeignKey('Watershed.watershed_id', onupdate="cascade"),
                          nullable=False)
    basin_id = Column(Integer, primary_key=True, nullable=False)
    basin_name = Column(String(250), nullable=False)
    shapefile = Column(types.LargeBinary, nullable=True)

    watershed = relationship('Watershed',
                             backref=backref('Basin',lazy='dynamic'))

class Results(Base):
    __tablename__ = 'Results'

    id = Column(Integer, primary_key=True)
    basin_id = Column(Integer, ForeignKey('Basin.basin_id', onupdate="cascade"), index=True)
    run_id = Column(Integer, ForeignKey('RunMetadata.run_id', onupdate="cascade"), index=True)
    date_time = Column(types.DateTime(), nullable=False, index=True)
    variable = Column(String(250), nullable=True, index=True)
    variable_id = Column(Integer, ForeignKey('VariableUnits.id', onupdate="cascade"), nullable=True)
    value = Column(types.Float(), nullable=True)
    elevation = Column(String(250), nullable=True)

    variable_units = relationship('VariableUnits',
                                  backref=backref('Results'))

    runmetadata = relationship('RunMetadata',
                               backref=backref('Results',lazy='dynamic'))

    basinid = relationship('Basin',
                               backref=backref('Results',lazy='dynamic'))

class VariableUnits(Base):
    __tablename__ = 'VariableUnits'

    id = Column(Integer, primary_key=True, autoincrement=True)
    run_id = Column(Integer, ForeignKey('RunMetadata.run_id', onupdate="cascade"),index=True)
    variable = Column(String(250), nullable=True)
    unit = Column(String(250), nullable=True)
    name = Column(String(250), nullable=True)

    run_metadata = relationship('RunMetadata',
                               backref=backref('VariableUnits',lazy='dynamic'))

# class Watersheds(object):
#     '''
#     To add a watershed:
#         'Basin Name':{'watershed_id':n+1,
#                              'watershed_name':'Basin Name short version',
#                              'basins': '',
#                              'shapefile':'empty'}
#
#     Also fill out Basins with relevant subbasins.
#
#     Basins main key and watershed_id must match a Watershed!
#
#     '''
#
#     watersheds = {
#                 'Boise River Basin':{'watershed_id':1,
#                                      'watershed_name':'Boise',
#                                      'basins': '',
#                                      'shapefile':'empty'},
#
#                 'Extended Tuolumne':{'watershed_id':2,
#                                      'watershed_name':'Extended Tuolumne',
#                                      'basins': '',
#                                      'shapefile':'empty'},
#
#                 'San Joaquin River Basin':{'watershed_id':3,
#                                            'watershed_name':'San Joaquin',
#                                            'basins': '',
#                                            'shapefile':'empty'},
#
#                 'Lakes Basin':{'watershed_id':4,
#                                'watershed_name':'Lakes',
#                                'basins': '',
#                                'shapefile':'empty'},
#
#                 'Merced River Basin':{'watershed_id':5,
#                                       'watershed_name':'Merced',
#                                       'basins': '',
#                                       'shapefile':'empty'},
#
#                 'Kaweah River Basin':{'watershed_id':6,
#                                       'watershed_name':'Kaweah',
#                                       'basins': '',
#                                       'shapefile':'empty'},
#
#                 'Kings River Basin':{'watershed_id':7,
#                                      'watershed_name':'Kings',
#                                      'basins': '',
#                                      'shapefile':'empty'},
#
#                 'Reynolds Creek':{'watershed_id':8,
#                                   'watershed_name':'Reynolds Creek',
#                                   'basins': '',
#                                   'shapefile':'empty'}
#                 }
#
#
# class Basins(object):
#     '''
#     This class defines the basin metadata for all of the subbasins that are
#     used in the database.
#
#     To add basins, Basins main key and watershed_id must match the
#     correct Watershed!
#
#     self.edges = np.arange(self.elev_bins[0],
#                            self.elev_bins[1]+self.elev_bins[2],
#                            self.elev_bins[2])
#
#     '''
#     basins = {
#                 # BRB
#                 'Boise River Basin':{'watershed_id':1,
#                                      'basin_id':1,
#                                      'basin_name':'Boise River Basin'},
#                 'Featherville':{'watershed_id':1,
#                                 'basin_id':2,
#                                 'basin_name':'Featherville'},
#                 'Twin Springs':{'watershed_id':1,
#                                 'basin_id':3,
#                                 'basin_name':'Twin Springs'},
#                 'Mores Creek':{'watershed_id':1,
#                                'basin_id':4,
#                                'basin_name':'Mores Creek'},
#
#                 # Tuolumne
#                 'Extended Tuolumne':{'watershed_id':2,
#                                      'basin_id':5,
#                                      'basin_name':'Extended Tuolumne',
#                                      'defaults':{'plotorder':['Tuolumne',
#                                                               'Cherry Creek',
#                                                               'Eleanor'],
#                                                   'edges':np.arange(4000,14000,1000),
#                                                   'dem_path':'/home/ops/wy2019/tuolumne/topo/topo.nc'}},
#                 'Tuolumne':{'watershed_id':2,
#                             'basin_id':6,
#                             'basin_name':'Tuolumne'},
#                 'Cherry Creek':{'watershed_id':2,
#                                 'basin_id':7,
#                                 'basin_name':'Cherry Creek'},
#                 'Eleanor':{'watershed_id':2,
#                            'basin_id':8,
#                            'basin_name':'Eleanor'},
#
#                 # San Joaquin
#                 'San Joaquin River Basin':{'watershed_id':3,
#                                            'basin_id':9,
#                                            'basin_name':'San Joaquin',
#                                            'defaults':{'plotorder':['San Joaquin River Basin',
#                                                                     'Main',
#                                                                     'Redinger',
#                                                                     'South Fork',
#                                                                     'Auberry'],
#                                                         'edges':np.arange(4000,14000,1000),
#                                                         'dem_path':'/home/ops/wy2019/sanjoaquin/topo/topo.nc'}},
#
#                 'South Fork':{'watershed_id':3,
#                               'basin_id':10,
#                               'basin_name':'South Fork'},
#
#                 'Main':{'watershed_id':3,
#                         'basin_id':11,
#                         'basin_name':'Main'},
#
#                 # Replaced
#                 'Jose Creek':{'watershed_id':3,
#                               'basin_id':12,
#                               'basin_name':'Jose Creek'},
#
#                 # Replaced
#                 'Willow Creek':{'watershed_id':3,
#                                 'basin_id':13,
#                                 'basin_name':'Willow Creek'},
#
#                 'Auberry':{'watershed_id':3,
#                                 'basin_id':27,
#                                 'basin_name':'Auberry'},
#
#                 'Redinger':{'watershed_id':3,
#                                 'basin_id':28,
#                                 'basin_name':'Redinger'},
#
#                 # Reynolds
#                 'Reynolds Creek':{'watershed_id':8,
#                                   'basin_id':14,
#                                   'basin_name':'Reynolds Creek'},
#                 'Tollgate':{'watershed_id':8,
#                             'basin_id':15,
#                             'basin_name':'Tollgate'},
#
#                 # Lakes
#                 'Lakes Basin':{'watershed_id':4,
#                                'basin_id':16,
#                                'basin_name':'Lakes',
#                                'defaults':{'plotorder':['Lakes'],
#                                             'edges':np.arange(9000,13000,1000),
#                                             'dem_path':'/home/ops/wy2019/lakes/topo/topo.nc'}},
#
#                 # Merced
#                 'Merced River Basin':{'watershed_id':5,
#                                       'basin_id':17,
#                                       'basin_name':'Merced',
#                                       'defaults':{'plotorder':['Merced River Basin',
#                                                           'Yosemite', 'Pohono',
#                                                           'Upper South Fork',
#                                                           'Lower South Fork',
#                                                           'El Portal', 'West'],
#                                                    'edges':np.arange(4000,14000,1000),
#                                                    'dem_path':'/home/ops/wy2019/merced/topo/topo.nc'}},
#
#                 'West':{'watershed_id':5,
#                                       'basin_id':36,
#                                       'basin_name':'West'},
#
#                 'Yosemite':{'watershed_id':5,
#                                       'basin_id':37,
#                                       'basin_name':'Yosemite'},
#
#                 'Upper South Fork':{'watershed_id':5,
#                                       'basin_id':38,
#                                       'basin_name':'Upper South Fork'},
#
#                 'El Portal':{'watershed_id':5,
#                                       'basin_id':39,
#                                       'basin_name':'El Portal'},
#
#                 'Lower South Fork':{'watershed_id':5,
#                                       'basin_id':40,
#                                       'basin_name':'Lower South Fork'},
#
#                 'Pohono':{'watershed_id':5,
#                                       'basin_id':41,
#                                       'basin_name':'Pohono'},
#
#                 # Kaweah
#                 'Kaweah River Basin':{'watershed_id':6,
#                                       'basin_id':18,
#                                       'basin_name':'Kaweah River Basin',
#                                       'defaults':{'plotorder':['Kaweah River Basin',
#                                                   'North Fork', 'Marble Fork',
#                                                   'Middle Fork', 'East Fork',
#                                                   'South Fork', 'Lake Kaweah',
#                                                   'Kaweah River'],
#                                                    'edges':np.arange(4000,14000,1000),
#                                                    'dem_path':'/home/ops/wy2019/kaweah/topo/topo.nc'}},
#
#                 'North Fork':{'watershed_id':6,
#                                       'basin_id':29,
#                                       'basin_name':'North Fork'},
#
#                 'Marble Fork':{'watershed_id':6,
#                                       'basin_id':30,
#                                       'basin_name':'Marble Fork'},
#
#                 'East Fork':{'watershed_id':6,
#                                       'basin_id':31,
#                                       'basin_name':'East Fork'},
#
#                 'South Fork':{'watershed_id':6,
#                                       'basin_id':32,
#                                       'basin_name':'South Fork'},
#
#                 'Middle Fork':{'watershed_id':6,
#                                       'basin_id':33,
#                                       'basin_name':'Middle Fork'},
#
#                 'Lake Kaweah':{'watershed_id':6,
#                                       'basin_id':34,
#                                       'basin_name':'Lake Kaweah'},
#
#                 'Kaweah River':{'watershed_id':6,
#                                       'basin_id':35,
#                                       'basin_name':'Kaweah River'},
#
#                 # Kings
#                 'Kings River Basin':{'watershed_id':7,
#                                      'basin_id':19,
#                                      'basin_name':'Kings River Basin',
#                                      'defaults':{'plotorder':['Kings River Basin',
#                                                  'South Fork', 'Middle Fork',
#                                                  'North Fork', 'Dinkey Creek',
#                                                  'Middle South Fork',
#                                                  'West Kings', 'Mill Creek'],
#                                                   'edges':np.arange(4000,14000,1000),
#                                                   'dem_path':'/home/ops/wy2019/kings/topo/topo.nc'}},
#
#                 'Middle Fork':{'watershed_id':7,
#                               'basin_id':20,
#                               'basin_name':'Middle Fork'},
#
#                 'West Kings':{'watershed_id':7,
#                               'basin_id':21,
#                               'basin_name':'West Kings'},
#
#                 'Middle South Fork':{'watershed_id':7,
#                               'basin_id':22,
#                               'basin_name':'Middle South Fork'},
#
#                 'South Fork':{'watershed_id':7,
#                               'basin_id':23,
#                               'basin_name':'South Fork'},
#
#                 'Mill Creek':{'watershed_id':7,
#                               'basin_id':24,
#                               'basin_name':'Mill Creek'},
#
#                 'North Fork':{'watershed_id':7,
#                               'basin_id':25,
#                               'basin_name':'North Fork'},
#
#                 'Dinkey Creek':{'watershed_id':7,
#                               'basin_id':26,
#                               'basin_name':'Dinkey Creek'}
#
#                 }




import numpy as np
import os
import copy
import pandas as pd
from datetime import datetime, timedelta
import logging
import coloredlogs
import netCDF4 as nc
from snowav.database.package_results import package
from snowav.database.database import delete, query
from snowav.utils.utilities import calculate, precip, snow_line, input_summary
from tablizer.defaults import Units
from tablizer.tablizer import get_existing_records
import time
import warnings

def process(args):
    '''
    Process AWSM model results made in awsm_daily format, remove any existing
    database results with the same run_name and date, and place results on
    database. See README.md and CoreConfig.ini for more information.

    Args
    ------
    args : dict

    Returns
    -------
    precip_total : array
    rain_total : array
    log : list
    density : array
    flag : bool

    '''

    pd.options.mode.chained_assignment = None
    warnings.filterwarnings('ignore')

    log = []
    cclimit = -5*1000*1000
    t = 0
    dem = args['dem']
    outputs = args['outputs']
    rundirs_dict = args['rundirs_dict']
    edges = args['edges']
    masks = args['masks']
    ixd = args['ixd']
    pixel = args['pixel']
    lbls = {}
    lbls['depthlbl'] = args['depthlbl']
    lbls['vollbl'] = args['vollbl']
    lbls['elevlbl'] = args['elevlbl']
    raw_out = ['swi_z','evap_z','snowmelt','swe_z','depth','density','coldcont']
    precip_path = '/smrfOutputs/precip.nc'
    proc_list = copy.deepcopy([*args['vars'].keys()])

    dz = pd.DataFrame(np.nan, index = edges, columns = masks.keys())
    precip_total = np.zeros((args['nrows'],args['ncols']))
    rain_total = np.zeros((args['nrows'],args['ncols']))

    for iters, out_date in enumerate(outputs['dates']):

        # assume to start that we are processing everything
        proc_list = copy.deepcopy([*args['vars'].keys()])

        pass_flag = False

        # delete anything on the database with the same run_name and date
        for bid in args['plotorder']:
            results = query(args['connector'], out_date, out_date,
                            args['run_name'], args['basins'], bid, 'swe_vol')

            if not results.empty:
                if not args['db_overwrite']:
                    if bid == args['plotorder'][0]:
                        logging.info(' Skipping outputs for {}, {}, '
                                     'database records exist...'.format(bid,
                                     out_date.strftime("%Y-%-m-%-d %H:00")))
                    else:
                        logging.debug(' Skipping outputs for {}, {}, '
                                     'database records exist...'.format(bid,
                                     out_date.strftime("%Y-%-m-%-d %H:00")))
                    pass_flag = True
                    proc_list = ['density']

                else:
                    out = delete(args['connector'], args['basins'], out_date,
                                 out_date, bid, args['run_name'])
                    if out != []:
                        if bid == args['plotorder'][0]:
                            logging.info(out[0])
                        else:
                            logging.debug(out[0])

        # Initialize output dataframes for every day
        daily_outputs = {'swe_vol':dz.copy(),
                         'swe_avail':dz.copy(),
                         'swe_unavail':dz.copy(),
                         'swe_z':dz.copy(),
                         'swi_vol':dz.copy(),
                         'swi_z':dz.copy(),
                         'precip_vol':dz.copy(),
                         'precip_z':dz.copy(),
                         'depth':dz.copy(),
                         'density':dz.copy(),
                         'rain_z':dz.copy(),
                         'evap_z':dz.copy(),
                         'coldcont':dz.copy(),
                         'snow_line':pd.DataFrame(0, index = ['total'],
                                                  columns = masks.keys())}

        density = {}
        for name in masks:
            density[name] = {}

        # Get daily rain from hourly input data
        flag, path, pre, rain = precip(rundirs_dict[outputs['time'][iters]],
                                       precip_path)

        if flag:
            precip_total = precip_total + pre
            rain_total = rain_total + rain

        else:
            logging.info(' WARNING! Expected to find {} but it is not a valid '
                       'file, precip will not be calculated or put on the '
                       'database, no precip figures will be made'.format(path))

        if args['inputs_flag']:
            mask_list = []
            for name in masks:
                mask_list.append(copy.deepcopy(masks[name]['mask']))

            variable = 'air_temp'
            methods = args['inputs_methods']
            percentiles = args['inputs_percentiles']
            location = args['connector']
            run_name = args['run_name']
            basins = args['basins']
            run_id = args['run_id']

            if os.path.splitext(location)[1] == '.db':
                db = 'sqlite'
            else:
                db = 'sql'

            sf = rundirs_dict[outputs['time'][iters]].replace('runs','data')
            sf = sf.replace('run','data') + '/smrfOutputs/'
            inputs = os.listdir(sf)

            df = get_existing_records(args['connector'], args['dbs'])
            df = df.set_index('date_time')
            df.sort_index(inplace=True)

            for input in inputs:
                input_path = os.path.join(sf,input)
                variable = os.path.splitext(input)[0]

                if variable in args['inputs_variables']:
                    for basin in args['inputs_basins']:

                        basin_id = int(basins[basin]['basin_id'])

                        dfs = df[((df['variable'] == variable) &
                                 (df['run_name'] == run_name) &
                                 (df['basin_id'] == basin_id))]
                        idx = dfs.index

                        if not idx.contains(out_date) or (idx.contains(out_date) and args['db_overwrite']):
                            if ((basin == args['inputs_basins'][0]) and
                                (variable == args['inputs_variables'][0])):
                                logging.info(' Processing inputs for {}, {}, '
                                             '{} '.format(variable, basin,
                                             out_date.strftime("%Y-%-m-%-d %H:00")))
                            else:
                                logging.debug(' Processing inputs for {}, {}, '
                                             '{} '.format(variable, basin,
                                             out_date.strftime("%Y-%-m-%-d %H:00")))

                            input_summary(input_path, variable, methods,
                                          percentiles, db, location, run_name,
                                          basin_id, run_id, masks[basin]['mask'])

                        if idx.contains(out_date) and not args['db_overwrite']:
                            if ((basin == args['inputs_basins'][0]) and
                                (variable == args['inputs_variables'][0])):
                                logging.info(' Skipping inputs for {}, {}, '
                                             '{}, database records exist...'
                                             ''.format(variable, basin,
                                             out_date.strftime("%Y-%-m-%-d %H:00")))
                            else:
                                logging.debug(' Skipping inputs for {}, {}, '
                                             '{}, database records exist...'
                                             ''.format(variable, basin,
                                             out_date.strftime("%Y-%-m-%-d %H:00")))

        swe = copy.deepcopy(outputs['swe_z'][iters])
        cold = copy.deepcopy(outputs['coldcont'][iters])

        # Loop over outputs (depths are copied, volumes are calculated)
        for k in proc_list:

            # Mask by subbasin
            for name in masks:
                if name == args['plotorder'][0]:
                    logging.info(' Processing {}, {}, {} '.format(k, name,
                                 rundirs_dict[outputs['time'][iters]].split('/')[-1]))
                else:
                    logging.debug(' Processing {}, {}, {} '.format(k, name,
                                 rundirs_dict[outputs['time'][iters]].split('/')[-1]))

                mask = copy.deepcopy(masks[name]['mask'])
                elevbin = ixd*mask

                snow_mask = swe > 0
                avail_mask = cold > cclimit
                unavail_mask = cold <= cclimit

                if k in raw_out:
                    o = copy.deepcopy(outputs[k][iters])

                # elevation band
                for n in np.arange(0,len(edges)):
                    b = edges[n]
                    elev_mask = elevbin == n

                    # list of masks
                    be = [mask, elev_mask]
                    ba = [mask, avail_mask]
                    bu = [mask, unavail_mask]
                    bes = [mask, elev_mask, snow_mask]
                    besa = [mask, elev_mask, snow_mask, avail_mask]
                    besu = [mask, elev_mask, snow_mask, unavail_mask]

                    if k == 'swe_z':
                        daily_outputs['swe_unavail'].loc[b,name] = \
                                    calculate(o, pixel, besu, 'sum','volume')
                        daily_outputs['swe_avail'].loc[b,name] = \
                                    calculate(o, pixel, besa, 'sum','volume')
                        daily_outputs['swe_vol'].loc[b,name] = \
                                    calculate(o, pixel, be, 'sum','volume')
                        daily_outputs['swe_z'].loc[b,name] = \
                                    calculate(o, pixel, be, 'mean','depth')

                    if k == 'swi_z':
                        daily_outputs[k].loc[b,name] = calculate(o,pixel,be,'mean','depth')
                        daily_outputs['swi_vol'].loc[b,name] = calculate(o,pixel,be,'sum','volume')

                    if k == 'depth':
                        daily_outputs[k].loc[b,name] = calculate(o, pixel, be, 'mean', 'snow_depth')

                    if k == 'evap_z':
                        daily_outputs[k].loc[b,name] = calculate(o, pixel, be, 'mean', 'depth')

                    if k == 'coldcont':
                        daily_outputs[k].loc[b,name] = calculate(o, pixel, bes, 'mean')

                    if k == 'density':
                        daily_outputs[k].loc[b,name] = calculate(o, pixel, bes, 'mean')

                        if out_date == outputs['dates'][-1]:
                            od = copy.deepcopy(o)
                            ml = [mask, elev_mask, snow_mask]
                            for m in ml:
                                m = m.astype('float')
                                m[m < 1] = np.nan
                                od = od * m

                            density[name][edges[n]] = copy.deepcopy(od)

                    if k == 'precip_z' and flag:
                        daily_outputs[k].loc[b,name] = calculate(pre, pixel, be, 'mean', 'depth')
                        daily_outputs['precip_vol'].loc[b,name] = calculate(pre, pixel, be, 'sum', 'volume')

                    if k == 'rain_z' and flag:
                        daily_outputs[k].loc[b,name] = calculate(rain, pixel, be, 'mean', 'depth')

                if k == 'evap_z':
                    daily_outputs[k].loc['total',name] = calculate(o, pixel, [mask, snow_mask], 'mean', 'depth')

                if k == 'coldcont':
                    daily_outputs[k].loc['total',name] = calculate(o, pixel, [mask, snow_mask], 'mean')

                if k == 'swe_z':
                    daily_outputs['swe_vol'].loc['total',name] = calculate(o, pixel, mask, 'sum','volume')
                    daily_outputs['swe_avail'].loc['total',name] = calculate(o, pixel, ba, 'sum','volume')
                    daily_outputs['swe_unavail'].loc['total',name] = calculate(o, pixel, bu, 'sum','volume')
                    daily_outputs[k].loc['total',name] = calculate(o, pixel, mask, 'mean','depth')
                    daily_outputs['snow_line'].loc['total',name] = snow_line(o, dem, mask, args['snow_limit'])

                if k == 'depth':
                    daily_outputs[k].loc['total',name] = calculate(o, pixel, mask, 'mean', 'snow_depth')

                if k == 'density':
                    daily_outputs[k].loc['total',name] = calculate(o, pixel, [mask, snow_mask], 'mean')

                if k == 'swi_z':
                    daily_outputs[k].loc['total',name] = calculate(o,pixel,mask,'mean','depth')
                    daily_outputs['swi_vol'].loc['total',name] = calculate(o,pixel,mask,'sum','volume')

                if k == 'precip_z' and flag:
                    daily_outputs[k].loc['total',name] = calculate(pre, pixel, mask, 'mean', 'depth')
                    daily_outputs['precip_vol'].loc['total',name] = calculate(pre, pixel, mask, 'sum', 'volume')

                if k == 'rain_z' and flag:
                    daily_outputs[k].loc['total',name] = calculate(rain, pixel, mask, 'mean', 'depth')

            df = daily_outputs[k].copy()
            df = df.round(decimals = 3)

            if not pass_flag:
                package(args['connector'], lbls, args['basins'], df, args['run_id'],
                        args['vid'], k, out_date, args['run_name'])

                # snow line
                if k == 'swe_z':
                    package(args['connector'], lbls, args['basins'], daily_outputs['snow_line'],
                            args['run_id'], args['vid'], 'snow_line', out_date, args['run_name'])

        hr = int(outputs['time'][iters])
        stamp = datetime.now().strftime("%Y-%-m-%-d %H:%M:%S")
        if not pass_flag:
            logging.info(' Completed {}, hours: {}, date: {}, at: {} UTC'.format(
                         rundirs_dict[outputs['time'][iters]].split('/')[-1],
                         str(hr - t), out_date.date().strftime("%Y-%-m-%-d"),
                         stamp))

        t = int(hr)

    return flag, log, precip_total, rain_total, density




from sys import exit
import argparse
import os
import coloredlogs
import logging
import pandas as pd
from snowav.framework.framework import snowav
from snowav.framework import framework
from snowav.framework.process_day import process
from snowav.plotting.swe_volume import swe_volume
from snowav.plotting.image_change import image_change
from snowav.plotting.plotlims import plotlims as plotlims
from snowav.utils.utilities import masks

def run():

    parser = argparse.ArgumentParser(description='Process AWSM model results, '
                                     'put results on database, create figures, '
                                     'and make pdf report. For other features, '
                                     'options, and behavior see README.md '
                                     'and CoreConfig.ini.')

    parser.add_argument('-f', '--config_file', dest='config_file', type=str,
                        help='Path to snowav configuration file.')

    parser.add_argument('-end_date', '--end-date', dest='end_date', type=str,
                        help='End date that will override what is given in the '
                        'config file, intended for airflow application.')

    parser.add_argument('-t', '--topo_path', dest='topo_path', type=str,
                        help='Path to topo.nc file.')

    parser.add_argument('-d', '--nc_path', dest='nc_path', type=str,
                        help='Path to single snow.nc file for swe '
                        'volume calculation and figure without database '
                        'processing.')

    parser.add_argument('-v', '--value', dest='value', type=str,
                        default='swe_vol',
                        choices=['swe_z','swe_vol','swe_avail','swe_unavail'],
                        help='SWE value to calculate for single snow.nc file.')

    parser.add_argument('-s', dest='show', action='store_true',
                        help='Flag to display single snow.nc calculation.')

    parser.add_argument('-no-s', dest='show', action='store_false')

    parser.set_defaults(show=False)

    parser.add_argument('-A', '--snow_a', dest='snow_a', type=str,
                        help='Path to "A" snow.nc file for simple difference, '
                        'must also include "-B", and must not use "-d".')

    parser.add_argument('-B', '--snow_b', dest='snow_b', type=str,
                        help='Path to "B" snow.nc file for simple difference, '
                        'must also include "-A", and must not use "-d".')

    parser.add_argument('-p', '--figs_path', dest='figs_path', type=str,
                        default=os.getcwd() + '/',
                        help='Path to save the figure, default is ./snowav/')

    args = parser.parse_args()

    ###########################################################################
    #  standard snowav processing                                             #
    ###########################################################################

    if args.config_file is not None:
        if not os.path.isfile(args.config_file):
            raise Exception('Config file {} does not '
                            'exist'.format(args.config_file))

        if args.end_date is not None:
            try:
                end_date = pd.to_datetime(args.end_date)
            except:
                raise Exception('pandas failed parsing {} to datetime'.format(
                    args.end_date))

        else:
            end_date = None

        snowav(config_file = args.config_file, end_date = end_date)
        exit()

    ###########################################################################
    #  single snow.nc file, or simple difference, no database                 #
    ###########################################################################
    if args.topo_path is None:
        raise Exception('Must use -t <topo.nc> argument for command line tools')

    if not os.path.isdir(args.figs_path):
        raise Exception('{} not a valid directory'.format(args.figs_path))

    if not os.path.isfile(args.topo_path):
        raise Exception('{} not a valid topo file'.format(args.topo_path))

    # Must use either single snow.nc, or difference between two snow.nc
    if args.nc_path is not None:
        nc_path = [os.path.abspath(args.nc_path)]
        diff = False
        if not os.path.isfile(nc_path[0]):
            raise Exception('{} not a valid file name'.format(nc_path[0]))

    elif (args.snow_a is not None) and (args.snow_b is not None):
        nc_path = [os.path.abspath(args.snow_a)] + [os.path.abspath(args.snow_b)]
        diff = True

        for path in nc_path:
            if not os.path.isfile(path):
                raise Exception('{} not a valid file name'.format(path))

    else:
        parser.error('Must use either -d, or both -A and -B arguments')

    log = logging.getLogger(__name__)
    level = 'INFO'
    levelname = 'INFO'
    coloredlogs.install(fmt='%(levelname)-5s %(message)s', level=level,
                        level_styles={'info':{'color':'green'}},logger=log)

    results = process(nc_path, os.path.abspath(args.topo_path), args.value, log)

    barcolors = ['xkcd:cobalt', 'xkcd:mustard green', 'xkcd:lichen',
                 'xkcd:pale green', 'xkcd:blue green', 'xkcd:bluish purple',
                 'xkcd:lightish purple', 'xkcd:deep magenta']

    fargs = {'show': args.show,
             'print':False,
             'masks':results['masks'],
             'barcolors':barcolors,
             'plotorder':results['plotorder'],
             'lims':plotlims(results['plotorder']),
             'vollbl':'TAF',
             'elevlbl':'ft',
             'depthlbl':'in',
             'edges':results['edges'],
             'labels':results['labels'],
             'percent_max':99.5,
             'percent_min':0.05,
             'figsize': (10,5),
             'dpi':250,
             'dplcs':1,
             'xlims':(0,len(results['edges'])),
             'figs_path':os.path.abspath(args.figs_path),
             'directory':'cli'}

    # difference between two snow.nc files
    if diff:
        if len(nc_path[0]) > 20:
            title = nc_path[1][-20::] + ' - ' + nc_path[0][-20::]
        else:
            title = nc_path

        df = results['df'][nc_path[1]] - results['df'][nc_path[0]]
        fargs['df'] = df
        fargs['image'] = results['outputs']['swe_z'][1] \
                         - results['outputs']['swe_z'][0]
        fargs['title'] = title

        log.info('Difference generated from:\n      {} '
                 'subtract\n      {}'.format(nc_path[1],nc_path[0]))
        log.info('Saved figure in {}'.format(os.path.abspath(args.figs_path)))

        image_change(fargs, None)
        exit()

    # snow.nc swe volume
    if not diff:
        if len(args.nc_path) > 35:
            title = '...' + args.nc_path[-20::]
        else:
            title = args.nc_path

        fargs['image'] = results['outputs']['swe_z'][0]/25.4
        fargs['title'] = '{}, {}'.format(args.value,title)
        fargs['df'] = results['df'][os.path.abspath(args.nc_path)]

        swe_volume(fargs, None)
        log.info('Saved figure in {}'.format(args.figs_path))

def can_i_snowav(config_file):
    '''
    Function that wraps snowav run in try/except for test case.

    Args
    ------
    config_file : str
        path to config file
    '''

    try:
        snowav(config_file = config_file)
        return True

    except Exception as e:
        print(e)
        return False

if __name__ == '__main__':
    run()




import os
import netCDF4 as nc
from spatialnc import ipw

def get_topo_stats(fp, filetype='netcdf'):
    """
    Get stats about topo from the topo file
    Returns:
        ts - dictionary of topo header data
    """

    fp = os.path.abspath(fp)

    ts = {}

    if filetype == 'netcdf':
        ds = nc.Dataset(fp, 'r')
        # ts['units'] = ds.variables['y'].units
        ts['y'] = ds.variables['y'][:]
        ts['x'] = ds.variables['x'][:]
        ts['nx'] = len(ts['x'])
        ts['ny'] = len(ts['y'])
        ts['du'] = ts['y'][1] - ts['y'][0]
        ts['dv'] = ts['x'][1] - ts['x'][0]
        ts['v'] = ts['x'][0]
        ts['u'] = ts['y'][0]
        ds.close()

    if filetype == 'ipw':
        i = ipw.IPW(fp)
        ts['nx'] = i.nsamps
        ts['ny'] = i.nlines
        ts['units'] = i.bands[0].units
        ts['du'] = i.bands[0].dline
        ts['dv'] = i.bands[0].dsamp
        ts['v'] = float(i.bands[0].bsamp)
        ts['u'] = float(i.bands[0].bline)
        ts['csys'] = i.bands[0].coord_sys_ID

    return ts




import os
from snowav.framework.query import query
from snowav.framework.read_config import read_config
from snowav.framework.process import process
from snowav.framework.parse import parse
from snowav.framework.figures import figures
from snowav.report.report import report
from snowav.database.database import run_metadata
from snowav.inflow.inflow import excel_to_csv
from datetime import datetime

class snowav(object):

    def __init__(self, config_file = None, external_logger = None, awsm = None,
                 end_date = None):
        '''
        Read config file, parse config options, process results, put results
        on database, make figures, and make pdf report.

        Args
        -----
        config_file : str
            snowav config file
        external_logger : object
            awsm logger
        awsm : class
            awsm class if being run in awsm

        '''

        # Get config options
        if awsm is None and os.path.isfile(config_file):
            self.config_file = config_file
            read_config(self, end_date = end_date)

        elif awsm is not None:
            self.config_file = awsm.configFile
            read_config(self, awsm = awsm, end_date = end_date)

        else:
            raise Exception('No config instance passed, or config file does '
                            'not exist!')

        # query existing database without processing
        if self.query_flag:
            query(self)

        # parse config options
        parse(self)

        # put run metadata on database
        run_metadata(self, self.run_name)

        # process results
        self.pargs['run_id'] = self.run_id
        self.pargs['vid'] = self.vid
        self.precip_flag, out, pre, rain, density = process(self.pargs)

        # gather process() outputs
        # for log in out:
        #     self._logger.info(log)

        self.density = density
        self.rain_total = rain
        self.precip_total = pre

        if not self.precip_flag:
            self.precip_depth_flag = False
            self.precip_validate_flag = False

        if self.inflow_flag and self.inflow_data is not None:
            args = {}
            args['path'] = self.inflow_data
            args['csv_file'] = self.summary_csv
            args['basin_headings'] = self.basin_headings
            args['inflow_headings'] = self.inflow_headings
            args['file_base'] = self.file_base
            args['sheet_name'] = self.sheet_name
            args['skiprows'] = self.skiprows
            args['date_idx'] = self.date_idx
            args['wy'] = self.wy
            args['overwrite'] = self.overwrite
            args['convert'] = self.convert

            excel_to_csv(args, self._logger)

        figures(self)

        # Do additional processing and figures if forecast is supplied. Some
        # field will be overwritten during forecast processing
        if self.forecast_flag:
            self._logger.info(' Starting forecast processing...')

            run_metadata(self, self.for_run_name)

            self.pargs['run_id'] = self.run_id
            self.pargs['vid'] = self.vid
            flags, out, pre, rain, density = process(self.pargs)

            for log in out:
                self._logger.info(log)

            self.density = density
            self.rain_total = rain
            self.precip_total = pre

            # figures for forecast run
            figures(self)

        if self.report_flag:
            report(self)

        elapsed = str(datetime.now() - self.proc_time_start)

        self._logger.info(' Completed snowav processing, elapsed time: {}'.format(elapsed))




import numpy as np
from matplotlib import pyplot as plt
import seaborn as sns
import utm
import netCDF4 as nc
import mysql.connector
import pandas as pd
from datetime import datetime
import os
from snowav.utils.stats import nashsutcliffe
import snowav.framework.figures

def stn_validate(args, logger):
    '''
    SWE validation at snow pillow sites.

    Requires proper config options for weather database connection and figure
    creation, see CoreConfig.ini for more information.

    Args
    ----------
    args : dict
        dictionary with required inputs, see swi() figure for more information.
    logger : list

    Returns
    ------
    logger : list
    flag : bool
        False if there are errors, will be used to remove stn_validate() figure
        from the report
    '''

    logger.debug(' Beginning stn_validate()...')

    rundirs = args['dirs']
    stns = args['stns']
    lbls = args['lbls']
    client = args['client']
    factor = args['factor']
    end_date = args['end_date']
    tbl = args['tbl']
    var = args['var']
    st_time = '{}-10-1 00:00:00'.format(str(args['wy'] - 1))
    end_time = end_date.date().strftime("%Y-%-m-%-d")
    ncxvec = args['snow_x']
    ncyvec = args['snow_y']
    stns = args['stns']
    fig_name_short = 'validation_'
    flag = True
    px = args['px']
    py = args['py']

    stns_pixel = []
    for sta in stns:
        stns_pixel = stns_pixel + ['{}_'.format(sta) + str(s) for s in range(0,9)]

    measure = pd.DataFrame(index = pd.date_range(datetime(args['wy']-1,10,1),
                            end_date,freq='D'), columns = stns)

    model = pd.DataFrame(index = pd.date_range(datetime(args['wy']-1,10,1),
                         end_date,freq='D'), columns = stns_pixel)

    qry = ('SELECT tbl_metadata.* FROM tbl_metadata INNER JOIN tbl_stations ON '
          'tbl_metadata.id = tbl_stations.metadata_id WHERE tbl_stations.client'
          '="{}";'.format(client))

    cnx = mysql.connector.connect(user=args['user'], password=args['password'],
                                  host=args['host'], port=args['port'],
                                  database='weather_db')

    metadata = pd.read_sql(qry, cnx)

    if metadata.empty:
        flag = False
        return

    metadata.index = metadata['primary_id']
    metadata = metadata[~metadata.index.duplicated(keep='first')]

    # station results
    for iters,stn in enumerate(stns):
        cnx = mysql.connector.connect(user=args['user'],password=args['password'],
                                      host=args['host'], port=args['port'],
                                      database='weather_db')

        qry = ('SELECT weather_db.{0}.date_time, weather_db.{0}.{1} FROM '
               'weather_db.{0} WHERE weather_db.{0}.date_time between "{2}" '
               'and "{3}" AND weather_db.{0}.station_id IN '
               ' ("{4}") ;'.format(args['tbl'],args['var'],st_time,end_time,stn))

        # data = pd.read_sql(qry, cnx, index_col=bytes(bytearray(b'date_time')))
        data = pd.read_sql(qry, cnx, index_col='date_time')
        data.index.names=['date_time']
        dind = pd.date_range(st_time,end_time,freq='D')
        measure[stn] = data.reindex(dind)

    sns.set_style('darkgrid')
    sns.set_context('notebook')

    plt.close(9)

    if len(stns) <= 3:
        fig, axs = plt.subplots(num=9,figsize=(6,6),dpi=args['dpi'],nrows=3,ncols=1)

    if (len(stns) > 3) and (len(stns) <= 6):
        fig, axs = plt.subplots(num=9,figsize=(10,10),dpi=args['dpi'],nrows=3,ncols=2)

    if (len(stns) > 6) and (len(stns) <= 9):
        fig, axs = plt.subplots(num=9,figsize=(10,10),dpi=args['dpi'],nrows=3,ncols=3)

    if (len(stns) > 9) and (len(stns) <= 12):
        fig, axs = plt.subplots(num=9,figsize=(10,10),dpi=args['dpi'],nrows=4,ncols=3)

    axs = axs.flatten()

    set_x_on = 12
    if len(stns) in (2,5,8):
        fig.delaxes(axs[len(stns)])
        if len(stns) == 5:
            set_x_on = 3
        if len(stns) == 8:
            set_x_on = 5

    for rname in rundirs:
        logger.debug(' Loading pixel values in {}...'.format(rname.split('runs')[-1]))

        # could add if args['ncfile'] == 'em.nc' change to data/smrf/precip
        snowfile = os.path.join(rname, args['ncfile'])

        if not os.path.isfile(snowfile):
            raise Exception('In stn_validate(), {} not a valid file'.format(snowfile))

        ncf = nc.Dataset(snowfile, 'r')
        t = nc.num2date(ncf.variables['time'][0],ncf.variables['time'].units)

        # Only load the rundirs that we need
        if (t.date() <= end_date.date()):

            for iters,stn in enumerate(stns):
                ll = utm.from_latlon(metadata.ix[stn,'latitude'],metadata.ix[stn,'longitude'])
                xind = np.where(abs(ncxvec-ll[0]) == min(abs(ncxvec-ll[0])))[0]
                yind = np.where(abs(ncyvec-ll[1]) == min(abs(ncyvec-ll[1])))[0]

                if ((xind == 0) or (xind >= len(ncxvec)-1) or
                   (yind == 0) or (yind >= len(ncyvec)-1)):
                    logger.info('Station {} in [validate] stations is '
                                'outside of domain, exiting stn_validate() '
                                'and not generating figure'.format(stn))
                    flag = False
                    return '', flag

                for ix, (n,m) in enumerate(zip(px,py)):
                    land_stn = stn + '_{}'.format(str(ix))
                    swe = ncf.variables['specific_mass'][:,yind+m,xind+n][0][0][0]
                    model.loc[t.date(),land_stn] = swe

                    if (n == 0 and m == 0) and (rname == rundirs[-1]):
                        lbl = 'measured'
                        lblm = 'modeled'
                    else:
                        lbl = '__nolabel__'
                        lblm = 'modeled'

                    axs[iters].plot(measure[stn]/factor,'k',label=lbl)
                    axs[iters].plot(model[land_stn]/factor,'b',linewidth = 0.75,label=lblm)
                    axs[iters].set_title(lbls[iters])
                    axs[iters].set_xlim((datetime(args['wy']-1, 10, 1),end_date))

            ncf.close()

        else:
            ncf.close()

    # Nash-Sutcliffe on pixel 0,0
    if args['nash_sut_flag']:
        for iters,sta in enumerate(stns):
            pz_sta = sta + '_5'
            stime = datetime(args['wy']-1,10,15)
            nsv = nashsutcliffe(measure[measure.index>stime][stn].values.tolist(),
                                model[model.index>stime][pz_sta].values.tolist())
            # nstr = '{} Nash-Sutcliffe:\n{}'.format(sta,str(np.round(nsv,2)))
            nstr = 'Nash-Sutcliffe:\n{}'.format(str(np.round(nsv,2)))

            nsx = 0.05
            nsy = 0.85

            if iters == 0:
               nsy = 0.675

            if nsv > 0:
                axs[iters].text(nsx,nsy,nstr,horizontalalignment='left',
                                transform=axs[iters].transAxes,fontsize = 8)

    if len(stns) <= 6:
        for n in (1,3,5):
            axs[n].yaxis.tick_right()

        for n in (4,5):
            for tick in axs[n].get_xticklabels():
                tick.set_rotation(30)
        for n in (0,1,2,3):
            if n != set_x_on:
                axs[n].set_xticklabels('')
            else:
                for tick in axs[n].get_xticklabels():
                    tick.set_rotation(30)

    if (len(stns) > 6) and (len(stns) <=9):
        for n in (2,5,8):
            axs[n].yaxis.tick_right()

        for n in (6,7,8):
            for tick in axs[n].get_xticklabels():
                tick.set_rotation(30)

        for n in (0,1,2,3,4,5):
            axs[n].set_xticklabels('')

        for n in (1,4,7):
            axs[n].set_yticklabels('')

    if (len(stns) > 9) and (len(stns) <=12):
        for n in (2,5,8,11):
            axs[n].yaxis.tick_right()

        for n in (9,10,11):
            for tick in axs[n].get_xticklabels():
                tick.set_rotation(30)
        for n in (0,1,2,3,4,5,6,7,8):
            axs[n].set_xticklabels('')

    # Plot
    measure = measure.replace('[]',np.nan)
    maxm = np.nanmax(measure.max().values/factor)
    maxi = np.nanmax(model.max().values/factor)

    if maxm > maxi:
        maxswe = maxm
    else:
        maxswe = maxi

    for iters in range(0,len(stns)):
        axs[iters].set_ylim((-0.1,maxswe + maxswe*0.05))

    axs[0].legend(['measured','modeled'],loc='upper left')
    axs[0].set_ylabel('SWE [in]')

    plt.suptitle('SWE Validation at Snow Pillow Sites')
    plt.subplots_adjust(top=0.92)

    fig_name = '{}{}{}.png'.format(args['figs_path'],fig_name_short,args['directory'])
    if logger is not None:
        logger.info(' saving {}'.format(fig_name))

    snowav.framework.figures.save_fig(fig, fig_name)

    return fig_name_short, flag




import os
import numpy as np
import pandas as pd
import copy
from datetime import datetime, timedelta
from tablizer.tablizer import get_existing_records
from snowav.plotting.swi import swi
from snowav.plotting.basin_total import basin_total
from snowav.plotting.cold_content import cold_content
from snowav.plotting.compare_runs import compare_runs
from snowav.plotting.density import density
from snowav.plotting.flt_image_change import flt_image_change
from snowav.plotting.image_change import image_change
from snowav.plotting.precip_depth import precip_depth
from snowav.plotting.stn_validate import stn_validate
from snowav.plotting.write_properties import write_properties
from snowav.plotting.swe_volume import swe_volume
from snowav.plotting.inputs import inputs
from snowav.inflow.inflow import inflow
from snowav.plotting.diagnostics import diagnostics
from snowav.plotting.plotlims import plotlims
from snowav.plotting.point_values import point_values
from snowav.database.database import collect
from snowav.plotting.plotlims import plotlims as plotlims
import matplotlib as mpl

if os.environ.get('DISPLAY','') == '':
    mpl.use('Agg')

def figures(self):
    '''
    Set up and call snowav figures. See CoreConfig.ini and README.md for more on
    config options and use.

    Notes:
    - swe_volume() must be called before cold_content() if you want to use
    the same ylims for each.

    '''

    args = {'report_start':self.report_start.date().strftime("%Y-%-m-%-d"),
            'report_date':self.report_date.date().strftime("%Y-%-m-%-d"),
            'print':self.print_args_dict,
            'run_name':self.run_name,
            'start_date':self.start_date,
            'end_date':self.end_date,
            'directory':self.directory,
            'figs_path':self.figs_path,
            'edges':self.edges,
            'plotorder':self.plotorder,
            'labels':self.labels,
            'lims':plotlims(self.plotorder),
            'masks':self.masks,
            'figsize':self.figsize,
            'dpi':self.dpi,
            'depthlbl':self.depthlbl,
            'vollbl':self.vollbl,
            'elevlbl':self.elevlbl,
            'dplcs':self.dplcs,
            'barcolors':self.barcolors,
            'xlims':self.xlims,
            'depth_clip':self.depth_clip,
            'percent_min':self.percent_min,
            'percent_max':self.percent_max,
            'basins':self.basins,
            'wy':self.wy,
            'flag':False,
            'flt_flag':self.flt_flag}

    if self.flt_flag:
        args['flight_dates'] = self.flight_diff_dates

    fig_names = {}
    connector = self.connector

    ##########################################################################
    #       For each figure, collect 2D array image, by-elevation            #
    #       DataFrame, and set any figure-specific args inputs               #
    ##########################################################################
    if self.flt_flag:
        args['depth_factor'] = self.depth_factor
        args['update_file'] = self.update_file
        args['update_numbers'] = self.update_numbers
        args['flight_outputs'] = self.flight_outputs
        args['pre_flight_outputs'] = self.pre_flight_outputs
        args['connector'] = connector

        self.flight_diff_fig_names, self.flight_delta_vol_df = flt_image_change(args, self._logger)

        if self.flight_diff_fig_names == []:
            self.flt_flag = False

    if self.swi_flag:
        image = np.zeros_like(self.outputs['swi_z'][0])
        for n in range(self.ixs,self.ixe):
            image = image + self.outputs['swi_z'][n]*self.depth_factor

        df = collect(connector, args['plotorder'], args['basins'],
                     args['start_date'], args['end_date'], 'swi_vol',
                     args['run_name'], args['edges'], 'sum')

        args['df'] = df
        args['image'] = image
        args['title'] = 'Accumulated SWI\n{} to {}'.format(
                        self.start_date.date().strftime("%Y-%-m-%-d"),
                        self.end_date.date().strftime("%Y-%-m-%-d"))

        fig_names['swi'] = swi(args, self._logger)

    if self.image_change_flag:
        image = self.outputs['swe_z'][self.ixe] - self.outputs['swe_z'][self.ixs]

        start = collect(connector, args['plotorder'], args['basins'],
                        args['start_date'], args['start_date'],'swe_vol',
                        args['run_name'], args['edges'],'end')
        end = collect(connector, args['plotorder'], args['basins'],
                      args['start_date'], args['end_date'], 'swe_vol',
                      args['run_name'], args['edges'],'end')

        df = end - start

        args['df'] = df
        args['image'] = image*self.depth_factor
        args['title'] = 'Change in SWE Depth\n{} to {}'.format(
                        args['report_start'], args['report_date'])

        fig_names['image_change'] = image_change(args, self._logger)

    if self.swe_volume_flag:
        df = collect(connector, args['plotorder'], args['basins'],
                     args['start_date'], args['end_date'],'swe_vol',
                     args['run_name'], args['edges'],'end')

        image = self.outputs['swe_z'][self.ixe]*self.depth_factor

        args['df'] = df
        args['image'] = image
        args['title'] = 'SWE {}'.format(args['report_date'])

        fig_names['swe_volume'], args['ylims'] = swe_volume(args, self._logger)

    if self.cold_content_flag:
        df = collect(connector, args['plotorder'], args['basins'],
                     args['start_date'], args['end_date'], 'swe_unavail',
                     args['run_name'], args['edges'],'end')

        swe = self.outputs['swe_z'][self.ixe]
        image = self.outputs['coldcont'][self.ixe]*0.000001

        args['df'] = df
        args['swe'] = swe
        args['image'] = image
        args['title'] = 'Cold Content {}'.format(args['report_date'])

        fig_names['cold_content'] = cold_content(args, self._logger)

    if self.density_flag:
        image = self.outputs['density'][self.ixe]

        # self.density is assigned in process()
        args['density'] = self.density
        args['image'] = image
        args['title'] = 'Density {}'.format(args['report_date'])

        fig_names['density'] = density(args, self._logger)

    if self.basin_total_flag:
        wy_start = datetime(self.wy-1,10,1)
        swi_summary = collect(connector, args['plotorder'], args['basins'],
                              wy_start,args['end_date'],'swi_vol',
                              args['run_name'],'total','daily')
        df_swe = collect(connector, args['plotorder'], args['basins'],
                              wy_start,args['end_date'],'swe_vol',
                              args['run_name'],'total','daily')
        df_swi = swi_summary.cumsum()

        args['swi_summary'] = df_swi
        args['swe_summary'] = df_swe
        args['forecast_flag'] = self.forecast_flag
        args['flt_flag'] = self.flt_flag

        fig_names['basin_total'] = basin_total(args, self._logger)

    if self.precip_depth_flag:
        swi_image = np.zeros_like(self.outputs['swi_z'][0])
        for n in range(self.ixs,self.ixe):
            swi_image = swi_image + self.outputs['swi_z'][n]*self.depth_factor

        swi_df = collect(connector, args['plotorder'], args['basins'],
                         args['start_date'], args['end_date'], 'swi_z',
                         args['run_name'], args['edges'], 'sum')
        precip_df = collect(connector, args['plotorder'], args['basins'],
                            args['start_date'], args['end_date'], 'precip_z',
                            args['run_name'], args['edges'], 'sum')
        rain_df = collect(connector, args['plotorder'], args['basins'],
                          args['start_date'], args['end_date'], 'rain_z',
                          args['run_name'], args['edges'], 'sum')

        args['swi_image'] = swi_image
        args['precip_image'] = self.precip_total*self.depth_factor
        args['rain_image'] = self.rain_total*self.depth_factor
        args['swi_df'] = swi_df
        args['precip_df'] = precip_df
        args['rain_df'] = rain_df
        args['title'] = 'Depth of SWI, Precipitation, and Rain\n{} to {}'.format(
                        args['report_start'],args['report_date'])

        fig_names['precip_depth'] = precip_depth(args, self._logger)

    if self.diagnostics_flag:
        wy_start = datetime(self.wy-1,10,1)
        precip = collect(connector, args['plotorder'], args['basins'],
                         wy_start, args['end_date'], 'precip_z',
                         args['run_name'], args['edges'], 'daily')
        precip_per = collect(connector, args['plotorder'], args['basins'],
                         args['start_date'], args['end_date'], 'precip_z',
                         args['run_name'], args['edges'], 'daily')
        swe = collect(connector, args['plotorder'], args['basins'],
                         wy_start, args['end_date'], 'swe_z',
                         args['run_name'], args['edges'], 'daily')
        swe_per = collect(connector, args['plotorder'], args['basins'],
                         args['start_date'], args['end_date'], 'swe_z',
                         args['run_name'], args['edges'], 'daily')
        rho = collect(connector, args['plotorder'], args['basins'],
                         wy_start, args['end_date'], 'density',
                         args['run_name'], args['edges'], 'daily')
        rho_per = collect(connector, args['plotorder'], args['basins'],
                         args['start_date'], args['end_date'], 'density',
                         args['run_name'], args['edges'], 'daily')
        snow_line = collect(connector, args['plotorder'], args['basins'],
                         wy_start, args['end_date'], 'snow_line',
                         args['run_name'], args['edges'], 'daily')
        snow_line_per = collect(connector, args['plotorder'], args['basins'],
                         args['start_date'], args['end_date'], 'snow_line',
                         args['run_name'], args['edges'], 'daily')

        snow_line_per = snow_line_per.fillna(0)
        first_row = snow_line_per.iloc[[0]].values[0]
        snow_line_per = snow_line_per.apply(lambda row: row - first_row, axis=1)
        args['snow_line'] = snow_line
        args['snow_line_per'] = snow_line_per

        swe = swe.fillna(0)
        swe_per = swe_per.fillna(0)
        first_row = swe_per.iloc[[0]].values[0]
        swe_per = swe_per.apply(lambda row: row - first_row, axis=1)

        rho = rho.fillna(0)
        rho_per = rho_per.fillna(0)
        first_row = rho_per.iloc[[0]].values[0]
        rho_per = rho_per.apply(lambda row: row - first_row, axis=1)

        precip = precip.fillna(0)
        precip_per = precip_per.fillna(0)
        precip = precip.cumsum()
        precip_per = precip_per.cumsum()
        first_row = precip_per.iloc[[0]].values[0]
        precip_per = precip_per.apply(lambda row: row - first_row, axis=1)

        if self.diag_basins is None:
            args['dbasins'] = copy.deepcopy(self.plotorder)
        else:
            args['dbasins'] = self.diag_basins

        args['precip'] = precip
        args['precip_per'] = precip_per
        args['swe'] = swe
        args['swe_per'] = swe_per
        args['density'] = rho
        args['density_per'] = rho_per
        args['elevlbl'] = self.elevlbl

        diagnostics(args, self._logger)

    if self.stn_validate_flag:
        args['dirs'] = self.all_dirs
        args['stns'] = self.val_stns
        args['lbls'] = self.val_lbls
        args['client'] = self.val_client
        args['factor'] = 25.4
        args['user'] = self.wxdb_user
        args['password'] = self.wxdb_password
        args['host'] = self.wxdb_host
        args['port'] = self.wxdb_port
        args['snow_x'] = self.snow_x
        args['snow_y'] = self.snow_y
        args['stns'] = self.val_stns
        args['nash_sut_flag'] = self.nash_sut_flag

        # could change these for precip, px = 0, 'em.nc'
        args['tbl'] = 'tbl_level1'
        args['var'] = 'snow_water_equiv'
        args['px'] = (1,1,1,0,0,0,-1,-1,-1)
        args['py'] = (1,0,-1,1,0,-1,1,0,-1)
        args['ncfile'] = 'snow.nc'

        fig_names['valid'], flag = stn_validate(args, self._logger)

        if not flag:
            self.stn_validate_flag = False

    if self.point_values_flag and self.point_values:

        # check that self.point_values_date falls within options
        pv_date = self.point_values_date

        if pv_date is None:
            self._logger.info(' Value in [validate] point_values_date being '
                              'assigned to {}'.format(
                              self.end_date.date().strftime("%Y-%m-%d")))
            pv_date = self.end_date

        if pv_date < self.start_date or pv_date > self.end_date:
            self._logger.info(' Value in [validate] point_values_date outside '
                              'of range in [run] start_date - end_date, '
                              'point_values_date being assigned to {}'.format(
                              self.end_date.date().strftime("%Y-%m-%d")))
            pv_date = self.end_date
            idx = -1

        else:
            # get index for self.outputs for that date
            x = np.abs([date - pv_date for date in self.outputs['dates']])
            idx = x.argmin(0)

        model_date = self.outputs['dates'][idx]
        model_date = model_date.date().strftime('%Y-%m-%d')
        course_date = self.point_values_date.date().strftime('%Y-%m-%d')
        nsubplots = (self.point_values_settings[3]*self.point_values_settings[4]-1)-1

        for idxp, value in enumerate(self.point_values_properties):
            pflag = True
            df = pd.read_csv(self.point_values_csv[idxp])

            if self.point_values_heading[idxp] is not None:
                check_headings = [self.point_values_heading[idxp],'name',
                                  'latitude','longitude']
            else:
                check_headings = ['name', 'latitude', 'longitude']                                  

            for head in check_headings:
                if head not in df.columns.tolist():
                    self._logger.warn(' {} not in headings {} in {}'.format(
                                      head, df.columns.tolist(),
                                      self.point_values_csv[idxp]))
                    pflag = False

            if not pflag:
                continue

            if len(df.name.unique()) > nsubplots:
                self._logger.warn(' Number of subplots that will be generated in '
                                  'point_values() may not fit well with settings '
                                  'inpoint_values_settings, consider increasing '
                                  'nrows and/or ncols')

            fig_name = '{}model_pixel_{}_{}.csv'.format(self.figs_path,
                        value, self.end_date.date().strftime("%Y%m%d"))

            if value == 'swe_z':
                factor = self.depth_factor

            if value == 'density':
                factor = 1

            if value == 'depth':
                factor = 39.37

            array = self.outputs[value][idx]*factor

            point_values(array, value, df, (self.snow_x, self.snow_y), fig_name,
                         self.dem, self.figs_path, self.veg_type,
                         self.point_values_heading[idxp], model_date, course_date,
                         self.point_values_settings, self.pixel, self._logger)

    if self.compare_runs_flag:
        args['variables'] = ['swe_vol','swi_vol']
        wy_start = datetime(self.wy-1,10,1)

        if self.flt_flag:
            args['flag'] = True
        else:
            args['flag'] = False

        dict = {}
        for var in args['variables']:
            dict[var] = {}
            for run in self.compare_run_names:
                df = collect(connector, args['plotorder'][0], args['basins'],
                             wy_start,args['end_date'],var,run,'total','daily')
                if var == 'swi_vol':
                    df = df.cumsum()

                dict[var][run] = df

        args['dict'] = dict

        compare_runs(args, self._logger)

    if self.inflow_flag:
        wy_start = datetime(self.wy-1,10,1)
        swi_summary = collect(connector, args['plotorder'], args['basins'],
                              wy_start,args['end_date'],'swi_vol',
                              args['run_name'],'total','daily')
        df_swi = swi_summary.cumsum()

        args['swi_summary'] = df_swi

        if self.inflow_data is None:
            raw = pd.read_csv(self.summary_csv, skiprows = 1,
                              parse_dates=[0], index_col = 0)
            args['inflow_summary'] = raw[self.basin_headings]

        else:
            args['inflow_summary'] = pd.read_csv(self.summary_csv,
                                                 parse_dates=[0], index_col = 0)

        args['inflow_headings'] = self.inflow_headings
        args['basin_headings'] = self.basin_headings

        inflow(args, self._logger)

    if self.write_properties is not None:
        args['connector'] = self.connector
        args['wy_start'] = datetime(self.wy-1,10,1)

        write_properties(args, self.write_properties)

    if self.inputs_fig_flag:

        if self.mysql is not None:
            dbs = 'sql'
        else:
            dbs = 'sqlite'

        df = get_existing_records(connector, dbs)
        df = df.set_index('date_time')
        df.sort_index(inplace=True)

        ivalue = {}
        p = []

        for var in self.plots_inputs_variables:
            ivalue[var] = {}

            for basin in self.inputs_basins:
                bid = args['basins'][basin]['basin_id']
                ivalue[var][basin] = {}

                for func in self.inputs_methods:

                    if 'percentile' in func:
                        nfunc = '{}_{}'.format(func,str(self.inputs_percentiles[0]))

                        if ((var == self.plots_inputs_variables[0]) and
                             (basin == self.inputs_basins[0])):
                            p.append(nfunc)

                        ivalue[var][basin][nfunc] =  df[(df['function'] == nfunc) &
                                       (df['variable'] == var) &
                                       (df['basin_id'] == int(bid)) &
                                       (df['run_name'] == args['run_name'])]

                        nfunc = '{}_{}'.format(func,str(self.inputs_percentiles[1]))

                        if ((var == self.plots_inputs_variables[0]) and
                             (basin == self.inputs_basins[0])):
                            p.append(nfunc)

                        ivalue[var][basin][nfunc] =  df[(df['function'] == nfunc) &
                                       (df['variable'] == var) &
                                       (df['basin_id'] == int(bid)) &
                                       (df['run_name'] == args['run_name'])]
                    else:
                        ivalue[var][basin][func] =  df[(df['function'] == func) &
                                       (df['variable'] == var) &
                                       (df['basin_id'] == int(bid)) &
                                       (df['run_name'] == args['run_name'])]

                        if ((var == self.plots_inputs_variables[0]) and
                             (basin == self.inputs_basins[0])):
                            p.append(func)

        args['inputs'] = ivalue
        args['inputs_methods'] = p
        args['var_list'] = self.plots_inputs_variables
        args['inputs_basins'] = self.inputs_basins

        inputs(args, self._logger)

    if self.forecast_flag:
        print('Forecast figures in progress...')
        # if self.image_change_flag:
        #     image_change(self, forecast=self.for_run_name)
        #
        # if self.swi_flag:
        #
        #     if forecast is None:
        #         run_name = snow.run_name
        #         outputs = copy.deepcopy(snow.outputs)
        #         ixs = snow.ixs
        #         ixe = snow.ixe
        #         start_date = snow.start_date
        #         end_date = snow.end_date
        #         directory = snow.directory
        #         title = 'Accumulated SWI\n{} to {}'.format(
        #                                     snow.report_start.date().strftime("%Y-%-m-%-d"),
        #                                     snow.report_date.date().strftime("%Y-%-m-%-d"))
        #     swi(args)
        #
        # if self.swe_volume_flag:
        #     '''
        #         if day is not None:
        #             figs_path = day.figs_path
        #             name_append = 'day'
        #             date_stamp = day.date.strftime("%Y-%-m-%-d %H:%M") + ' (UTC)'
        #
        #     '''
        #
        #     args['directory'] = self.directory + '_forecast'
        #     args['run_name'] = self.for_run_name
        #     args['start_date'] = self.for_start_date
        #     args['end_date'] = self.for_end_date
        #     args['title'] = 'Forecast SWE \n {}'.format(self.for_end_date.date().strftime("%Y-%-m-%-d"))
        #
        #     swe = collect(self,args['plotorder'],args['start_date'],
        #                   args['end_date'],'swe_vol',args['run_name'],
        #                   args['edges'],'end')
        #
        #     image = self.for_outputs['swe_z'][self.for_ixe]*self.depth_factor
        #
        #     args['df'] = swe
        #     args['image'] = image
        #     args['title'] = 'SWE {}'.format(args['report_end'])
        #
        #     name, ylims = swe_volume(args, self._logger)
        #
        #     if self.basin_total_flag:
        #         '''
        #             for iter,d in enumerate(v['date_time'].values):
        #                 swe_summary.loc[d,bid] = v['value'].values[iter]
        #                 swi_summary.loc[d,bid] = v2['value'].values[iter]
        #
        #         swi_summary.sort_index(inplace=True)
        #
        #         # as a starting spot, add actual run
        #         swi_summary.iloc[0,:] = swi_summary.iloc[0,:] + swi_end_val.iloc[-1,:].values
        #         swi_summary = swi_summary.cumsum()
        #         '''
        #         # forecast True
        #         args['flag'] = True
        #
        #         basin_total(self, forecast=self.for_run_name)
        #
        #     if self.precip_depth_flag:
        #         precip_depth(self, forecast=self.for_run_name)

    self.fig_names = fig_names


def save_fig(fig, paths):
    '''
    Args
    ----------
    fig : object
        matplotlib figure object
    paths : list
        list of paths to save figure

    '''

    if type(paths) != list:
        paths = [paths]

    for path in paths:
        fig.savefig(path)




from snowav.utils.utilities import get_snowav_path
from inicheck.tools import get_user_config, check_config, cast_all_variables
from inicheck.output import generate_config, print_config_report
from inicheck.config import MasterConfig
import logging
import coloredlogs
import os
import pandas as pd
from datetime import datetime
import copy

def read_config(self, external_logger = None, awsm = None, end_date = None):
    '''
    Read snowav config file.

    '''

    print('Reading {} and loading files...'.format(self.config_file))

    snowav_mcfg = MasterConfig(modules = 'snowav')
    ucfg = get_user_config(self.config_file, mcfg=snowav_mcfg)
    ucfg.apply_recipes()
    ucfg = cast_all_variables(ucfg, ucfg.mcfg)
    self.snowav_path = get_snowav_path()

    warnings, errors = check_config(ucfg)
    if errors != [] or warnings != []:
        print_config_report(warnings, errors)

    self.tmp_log = []
    self.tmp_err = []
    self.tmp_warn = []

    self.proc_time_start = datetime.now()

    ####################################################
    #            snowav                                #
    ####################################################
    self.loglevel = ucfg.cfg['snowav']['log_level'].upper()
    self.log_to_file = ucfg.cfg['snowav']['log_to_file']
    self.save_path = ucfg.cfg['snowav']['save_path']
    self.units = ucfg.cfg['snowav']['units']
    self.filetype = ucfg.cfg['snowav']['filetype']
    self.elev_bins = ucfg.cfg['snowav']['elev_bins']
    self.directory = ucfg.cfg['snowav']['directory']
    self.dempath = ucfg.cfg['snowav']['dempath']
    self.run_name = ucfg.cfg['snowav']['run_name']
    self.plotorder = ucfg.cfg['snowav']['masks']
    self.plotlabels = ucfg.cfg['snowav']['plotlabels']

    ####################################################
    #           run                                    #
    ####################################################

    self.dplcs = ucfg.cfg['run']['decimals']
    self.start_date = ucfg.cfg['run']['start_date']
    self.end_date = ucfg.cfg['run']['end_date']

    if end_date is not None:
        self.end_date = end_date
        self.tmp_log.append(' Overriding config end_date with '
                            '{} given with snowav call'.format(end_date))

        if self.end_date <= self.start_date:
            raise Exception('end_date {} earlier than start_date {}'.format(
                            self.end_date, self.start_date))

    if self.start_date is not None and self.end_date is not None:
        self.start_date = self.start_date.to_pydatetime()
        self.end_date = self.end_date.to_pydatetime()

        if self.start_date >= self.end_date:
            self.tmp_log.append(' Error: [run] start_date >= [run] end_date')
            raise Exception('Error: [run] start_date >= [run] end_date')

    else:
        self.tmp_log.append(' [run] start_date and/or end_date was not '
                            'defined in config file, will be assigned '
                            'by available dates in directory')

    self.all_subdirs = ucfg.cfg['run']['all_subdirs']

    if (ucfg.cfg['run']['directory'] is None) and (awsm is not None):
        if self.all_subdirs is True:
            self.run_dirs = ([awsm.pathr + s for s in
                            os.listdir(awsm.pathr)
                            if (os.path.isdir(awsm.pathr + s)) ])
        else:
            self.run_dirs = awsm.pathr
            if type(self.run_dirs) != list:
                self.run_dirs = [self.run_dirs]

    else:
        directory = ucfg.cfg['run']['directory']

        if len(directory) == 1:
            directory = directory[0]

        if self.all_subdirs is True:
            self.run_dirs = ([directory + s for s in os.listdir(directory)
                            if (os.path.isdir(directory + s))])
        else:
            self.run_dirs = ucfg.cfg['run']['directory']
            if type(self.run_dirs) != list:
                self.run_dirs = [self.run_dirs]

    self.run_dirs.sort()

    ####################################################
    #         database
    ####################################################

    self.mysql = ucfg.cfg['database']['mysql']
    self.db_user = ucfg.cfg['database']['user']
    self.db_password = ucfg.cfg['database']['password']
    self.db_host = ucfg.cfg['database']['host']
    self.db_port = ucfg.cfg['database']['port']
    self.db_convert = ucfg.cfg['database']['convert_ws']
    self.add_basins = ucfg.cfg['database']['add_basins']
    self.db_overwrite = ucfg.cfg['database']['overwrite']

    if ((self.mysql is not None) and
        ((self.db_user is None) or
         (self.db_password is None) or
         (self.db_host is None) or
         (self.db_port is None)) ):
        raise Exception('If using config option [database] mysql, must also '
                        'supply user, password, host, and port')

    sqlite = ucfg.cfg['database']['sqlite']

    if (sqlite is not None) and os.path.isdir(os.path.dirname(sqlite)):
        self.sqlite = 'sqlite:///' + sqlite
    elif sqlite is None:
        self.sqlite = None

    if (sqlite is not None) and not os.path.isdir(os.path.dirname(sqlite)):
        raise Exception('Config option [database] sqlite: {} '.format(sqlite) +
                        'contains an invalid base path for the sqlite database')

    if self.mysql is not None and sqlite is not None:
        raise Exception('Config option [database] section contains both mysql '
                        'and sqlite entries, pick one...')

    ####################################################
    #           validate                               #
    ####################################################
    self.val_stns = ucfg.cfg['validate']['stations']
    self.val_lbls = ucfg.cfg['validate']['labels']
    self.val_client = ucfg.cfg['validate']['client']
    self.pre_val_stns = ucfg.cfg['validate']['pre_stations']
    self.pre_val_lbls = ucfg.cfg['validate']['pre_labels']
    self.wxdb_user = ucfg.cfg['validate']['user']
    self.wxdb_password = ucfg.cfg['validate']['password']
    self.wxdb_host = ucfg.cfg['validate']['host']
    self.wxdb_port = ucfg.cfg['validate']['port']
    self.point_values = ucfg.cfg['validate']['point_values']
    self.point_values_csv = ucfg.cfg['validate']['point_values_csv']
    self.point_values_date = ucfg.cfg['validate']['point_values_date']
    self.point_values_properties = ucfg.cfg['validate']['point_values_properties']
    self.point_values_heading = ucfg.cfg['validate']['point_values_heading']
    self.point_values_settings = ucfg.cfg['validate']['point_values_settings']

    if len(self.point_values_settings) != 14:
        self.tmp_log.append(' Expected [validate] point_values_settings to '
                            'have 14 values, point_values being set to False')
        self.point_values = False

    for n in range(0,10):
        self.point_values_settings[n] = int(self.point_values_settings[n])

    if type(self.point_values_properties) != list:
        self.point_values_properties = [self.point_values_properties]

    if type(self.point_values_heading) != list:
        self.point_values_heading = [self.point_values_heading]

    if type(self.point_values_csv) != list:
        self.point_values_csv = [self.point_values_csv]

    if self.point_values and self.point_values_csv is None:
        self.point_values = False
        self.tmp_log.append(' Config option [validate] point_values_csv '
                            'was not supplied, point_values being set '
                            'to False')

    if self.point_values and self.point_values_date is None:
        self.point_values = False
        self.tmp_log.append(' Config option [validate] point_values_date '
                            'was not supplied, point_values being set '
                            'to False')

    if not (len(self.point_values_csv) == len(self.point_values_properties) ==
            len(self.point_values_heading)):
        self.tmp_log.append(' Must supply the same number of [validate] '
                            'point_values_csv, point_values_properties, '
                            'and point_values_heading values, point_values '
                            'being set to False')


    ####################################################
    #           diagnostics                            #
    ####################################################
    self.diagnostics_flag = ucfg.cfg['diagnostics']['diagnostics']
    self.diag_basins = ucfg.cfg['diagnostics']['basins']
    self.diag_limit = ucfg.cfg['diagnostics']['limit']

    if self.diagnostics_flag:
        if self.diag_basins is not None:
            for basin in self.diag_basins:
                if basin not in self.plotorder:
                    self.tmp_log.append(' Config option [diagnostics] basin: {} '
                                        'does not match what was supplied in '
                                        '[snowav] masks: {}, diagnostics set '
                                        'to False'.format(basin, self.plotorder))
                    self.diagnostics_flag = False

    self.inputs_flag = ucfg.cfg['diagnostics']['inputs_table']
    self.inputs_variables = ucfg.cfg['diagnostics']['inputs_variables']
    self.inputs_percentiles = ucfg.cfg['diagnostics']['inputs_percentiles']
    self.inputs_methods = ucfg.cfg['diagnostics']['inputs_methods']
    self.inputs_basins = ucfg.cfg['diagnostics']['inputs_basins']

    # if self.inputs_flag:
    if self.inputs_basins is not None and self.plotorder is not None:
        for basin in self.inputs_basins:
            if basin not in self.plotorder:
                self.tmp_log.append(' Config option [diagnostics] '
                                    'inputs_basins: {} does not match what '
                                    'was supplied in [snowav] masks: {}, '
                                    'inputs set to '
                                    'False'.format(basin, plotorder))
                self.inputs_flag = False

    if self.inputs_flag:
        s = [x + ', ' for x in self.inputs_variables]
        self.tmp_log.append(' Using variables {} for inputs summary'.format(''.join(s)))

        s = [x + ', ' for x in self.inputs_methods]
        self.tmp_log.append(' Using methods {} for inputs summary'.format(''.join(s)))

    ####################################################
    #          plots                                   #
    ####################################################
    self.dpi = ucfg.cfg['plots']['dpi']
    self.depth_clip = ucfg.cfg['plots']['depth_clip']
    self.percent_min = ucfg.cfg['plots']['percent_min']
    self.percent_max = ucfg.cfg['plots']['percent_max']
    self.subs_fig = ucfg.cfg['plots']['subs_fig']
    self.density_flag = ucfg.cfg['plots']['density']
    self.swi_flag = ucfg.cfg['plots']['swi']
    self.current_image_flag = ucfg.cfg['plots']['current_image']
    self.image_change_flag = ucfg.cfg['plots']['image_change']
    self.cold_content_flag = ucfg.cfg['plots']['cold_content']
    self.swe_volume_flag = ucfg.cfg['plots']['swe_volume']
    self.basin_total_flag = ucfg.cfg['plots']['basin_total']
    self.stn_validate_flag = ucfg.cfg['plots']['stn_validate']
    self.nash_sut_flag = ucfg.cfg['plots']['disp_nash_sut']
    self.stns_file = ucfg.cfg['plots']['stns_file']
    self.precip_validate_flag = ucfg.cfg['plots']['precip_validate']
    self.inputs_fig_flag = ucfg.cfg['plots']['inputs']
    self.plots_inputs_variables = ucfg.cfg['plots']['inputs_variables']
    self.compare_runs_flag = ucfg.cfg['plots']['compare_runs']
    self.compare_run_names = ucfg.cfg['plots']['compare_run_names']
    self.compare_run_labels = ucfg.cfg['plots']['compare_run_labels']
    self.precip_depth_flag = ucfg.cfg['plots']['precip_depth']
    self.basin_detail_flag = ucfg.cfg['plots']['basin_detail']
    self.update_file = ucfg.cfg['plots']['update_file']
    self.print_args_dict = ucfg.cfg['plots']['print_args_dict']
    self.figsize = (ucfg.cfg['plots']['fig_length'],
                    ucfg.cfg['plots']['fig_height'])
    self.write_properties = ucfg.cfg['plots']['write_properties']
    self.point_values_flag = ucfg.cfg['plots']['point_values']
    if self.write_properties is not None and type(self.write_properties) != list:
        self.write_properties = [self.write_properties]

    numbers = ucfg.cfg['plots']['update_numbers']

    if numbers is not None:
        if type(numbers) != list:
            numbers = [numbers]
        self.update_numbers = [x - 1 for x in numbers]

    else:
        self.update_numbers = None

    if (self.compare_runs_flag and ((self.compare_run_names is None) or
        (self.compare_run_labels is None))):
        self.tmp_log.append(' Config option [plots] compare_runs set to True, '
                            'but one of compare_run_names or compare_run_labels '
                            'is empty, resetting compare_runs to False')
        self.compare_runs_flag = False

    if (self.compare_runs_flag and
        (len(self.compare_run_names) != len(self.compare_run_labels))):
        self.tmp_log.append(' Config option [plots] compare_runs set to True, '
                            'must supply equal length compare_run_names and  '
                            'compare_run_labels, resetting compare_runs to False')
        self.compare_runs_flag = False

    if self.update_file is not None:
        self.flt_flag = True

    else:
        self.flt_flag = False

    if (self.precip_validate_flag and ((self.val_client is None) or
       (self.pre_val_stns is None) or (self.pre_val_lbls is None))):
        self.tmp_log.append(' Config option [plots] precip_validate is being '
                            'set to False')

        self.precip_validate_flag = False

    if (self.stn_validate_flag and (self.val_client is None) or
       (self.val_stns is None) or (self.val_lbls is None) or
       (self.wxdb_user is None) or (self.wxdb_password is None) ):
        self.tmp_log.append(' Config option [plots] stn_validate is being '
                            'set to False')

        self.stn_validate_flag = False

    for var in self.plots_inputs_variables:
        if var not in self.inputs_variables:
            self.plots_inputs_variables.remove(var)
            self.tmp_log.append(' Config option [plots] inputs_variables '
                                'value {} not present in [diagnostics] '
                                'inputs_variables, being removed'.format(var))


    ####################################################
    #          report                                  #
    ####################################################
    self.report_flag = ucfg.cfg['report']['report']
    self.print_latex = ucfg.cfg['report']['print_latex']
    self.report_name = ucfg.cfg['report']['file']
    self.rep_title = ucfg.cfg['report']['title']
    self.rep_path = ucfg.cfg['report']['save_path']
    self.env_path = ucfg.cfg['report']['env_path']
    self.templ_path = ucfg.cfg['report']['templ_path']
    self.tex_file = ucfg.cfg['report']['tex_file']
    self.summary_file = ucfg.cfg['report']['summary']
    self.figs_tpl_path = ucfg.cfg['report']['figs_tpl_path']
    self.flight_figs = ucfg.cfg['report']['flight_figs']
    self.tables = ucfg.cfg['report']['tables']

    self.rep_swi_flag = ucfg.cfg['report']['swi']
    if not self.swi_flag:
        self.rep_swi_flag = False

    self.rep_image_change_flag = ucfg.cfg['report']['image_change']
    if not self.image_change_flag:
        self.rep_image_change_flag = False

    self.rep_cold_content_flag = ucfg.cfg['report']['cold_content']
    if not self.cold_content_flag:
        self.rep_cold_content_flag = False

    self.rep_swe_volume_flag = ucfg.cfg['report']['swe_volume']
    if not self.swe_volume_flag:
        self.rep_swe_volume_flag = False

    self.rep_basin_total_flag = ucfg.cfg['report']['basin_total']
    if not self.basin_total_flag:
        self.rep_basin_total_flag = False

    self.rep_stn_validate_flag = ucfg.cfg['report']['stn_validate']
    if not self.stn_validate_flag:
        self.rep_stn_validate_flag = False

    self.rep_compare_runs_flag = ucfg.cfg['report']['compare_runs']
    if not self.compare_runs_flag:
        self.rep_compare_runs_flag = False

    self.rep_precip_depth_flag = ucfg.cfg['report']['precip_depth']
    if not self.precip_depth_flag:
        self.rep_precip_depth_flag = False

    # check paths to see if they need default snowav path
    if self.env_path is None:
        self.env_path = os.path.abspath('./snowav/report/template/section_text')
    if self.templ_path is None:
        self.templ_path = os.path.abspath('./snowav/report/template')
    if self.summary_file is None:
        self.summary_file = os.path.abspath('./snowav/report/template/section_text/report_summary.txt')
    if self.tex_file is None:
        self.tex_file = os.path.abspath('./snowav/report/template/snowav_report.tex')
    if self.figs_tpl_path is None:
        self.figs_tpl_path = os.path.abspath('./snowav/report/figs')

    ####################################################
    #           forecast                               #
    ####################################################

    self.forecast_flag = ucfg.cfg['forecast']['report']

    if self.forecast_flag:
        self.for_start_date = ucfg.cfg['forecast']['start_date'].to_pydatetime()
        self.for_end_date = ucfg.cfg['forecast']['end_date'].to_pydatetime()
        self.for_run_name = ucfg.cfg['forecast']['run_name']

        if self.for_start_date >= self.for_end_date:
            self.tmp_log.append(' Error: config option [forecast] start_date > '
                                'end_date')
            raise Exception('Config option [forecast] start_date is greater '
                            'than end_date')

        self.for_run_dir = ([ucfg.cfg['forecast']['run_dir'] + s for s in
                        os.listdir(ucfg.cfg['forecast']['run_dir'])
                        if (os.path.isdir(ucfg.cfg['forecast']['run_dir'] + s)) ])

        self.for_run_dir.sort()

    ####################################################
    #           query                                  #
    ####################################################
    self.query_flag = ucfg.cfg['query']['query']
    self.q_basins = ucfg.cfg['query']['basins']
    self.q_value = ucfg.cfg['query']['value']
    self.q_run_name = ucfg.cfg['query']['run_name']
    self.q_print_all_runs = ucfg.cfg['query']['print_all_runs']
    self.q_start_date = ucfg.cfg['query']['start_date']
    self.q_end_date = ucfg.cfg['query']['end_date']
    self.q_total = ucfg.cfg['query']['total']
    self.q_output = ucfg.cfg['query']['output']
    self.q_csv_base_path = ucfg.cfg['query']['csv_base_path']
    self.q_database = ucfg.cfg['query']['database']

    ####################################################
    #           inflow                                 #
    ####################################################
    self.inflow_flag = ucfg.cfg['inflow']['inflow']
    self.inflow_data = ucfg.cfg['inflow']['inflow_data']
    self.summary_csv = ucfg.cfg['inflow']['summary_csv']
    self.inflow_headings = ucfg.cfg['inflow']['inflow_headings']
    self.basin_headings = ucfg.cfg['inflow']['basin_headings']
    self.sheet_name = ucfg.cfg['inflow']['sheet_name']
    self.skiprows = ucfg.cfg['inflow']['skiprows']
    self.overwrite = ucfg.cfg['inflow']['overwrite']
    self.file_base = ucfg.cfg['inflow']['file_base']
    self.date_idx = ucfg.cfg['inflow']['date_idx']
    self.convert = ucfg.cfg['inflow']['convert']

    self.ucfg = ucfg




import logging
import numpy as np
import os
import copy
import snowav.utils.get_topo_stats as ts
from snowav.utils.utilities import masks
from snowav.framework.outputs import outputs
import coloredlogs
import netCDF4 as nc
from collections import OrderedDict
from snowav.database.database import connect
from datetime import timedelta, datetime
from inicheck.output import print_config_report, generate_config
from snowav.utils.wyhr import handle_year_stradling, calculate_date_from_wyhr

def parse(self, external_logger=None):
    '''
    Parse options from config file after read_config().

    '''

    self.barcolors = ['xkcd:cobalt',
                      'xkcd:mustard green',
                      'xkcd:lichen',
                      'xkcd:pale green',
                      'xkcd:blue green',
                      'xkcd:bluish purple',
                      'xkcd:lightish purple',
                      'xkcd:deep magenta']

    # These are used in process() and database table VariableUnits
    self.vars = OrderedDict([('coldcont','cold content'),
                 ('evap_z','evaporation depth'),
                 ('rain_z','rain depth'),
                 ('density','density'),
                 ('depth','depth'),
                 ('precip_z','precipitation depth'),
                 ('precip_vol','precipitation volume'),
                 ('swi_z','surface water input depth'),
                 ('swi_vol','surface water input volume'),
                 ('swe_z','snow water equivalent depth'),
                 ('swe_vol','snow water equivalent volume'),
                 ('swe_avail','snow water equivalent available for melt'),
                 ('swe_unavail','snow water equivalent unavailable for melt')])

    out = masks(self.dempath, self.db_convert, plotorder = self.plotorder,
                plotlabels = self.plotlabels)

    self.dem = out['dem']
    self.veg_type = out['veg_type']
    self.masks = out['masks']
    self.nrows = out['nrows']
    self.ncols = out['ncols']
    self.plotorder = out['plotorder']
    self.labels = out['labels']

    for log in out['logger']:
        self.tmp_log.append(log)

    # Establish database connection
    self.basins, cnx, out = connect(sqlite = self.sqlite, sql = self.mysql,
                               plotorder = self.plotorder, user = self.db_user,
                               password = self.db_password, host = self.db_host,
                               port = self.db_port, convert = self.db_convert,
                               add = self.add_basins)
    self.connector = cnx

    for log in out:
        self.tmp_log.append(log)

    if self.loglevel == 'DEBUG':
        for basin in self.basins:
            self.tmp_log.append(' {}: {}'.format(basin, self.basins[basin]))

        self.tmp_log.append(' Connection: {}'.format(self.connector))

    # Check snow.nc file location, get topo stats and water year
    sfile = os.path.join(self.run_dirs[0],'snow.nc')

    if os.path.isfile(sfile):
        topo = ts.get_topo_stats(sfile, filetype = self.filetype)
        self.snow_x = topo['x']
        self.snow_y = topo['y']
        self.pixel = int(topo['dv'])

        ncf = nc.Dataset(sfile)
        t = nc.num2date(ncf.variables['time'][0],ncf.variables['time'].units)
        ncf.close()
        self.wy = handle_year_stradling(t) + 1

    else:
        print('\nGiven config options, expecting to find:\n   {}\nto load topo '
              'stats but is not a valid file\nCheck config [run] options, see '
              'CoreConfig.ini for details\n'.format(sfile))
        raise Exception('{} not a valid file'.format(sfile))

    # make the bins
    edges = np.arange(self.elev_bins[0],
                      self.elev_bins[1]+self.elev_bins[2],
                      self.elev_bins[2])

    # use for definition
    self.edges = np.arange(self.elev_bins[0]-self.elev_bins[2],
                           self.elev_bins[1],
                           self.elev_bins[2])

    if self.units == 'TAF':
        self.conversion_factor = ((self.pixel**2)*0.000000810713194*0.001)
        self.depth_factor = 0.03937
        self.dem = self.dem * 3.28
        self.depthlbl = 'in'
        self.vollbl = self.units
        self.elevlbl = 'ft'

    self.ixd = np.digitize(self.dem,edges)
    self.xlims = (0,len(edges))

    if self.loglevel == 'DEBUG' and self.log_to_file is not True:
        print('Reading files in {}...'.format(self.run_dirs[0].split('runs')[0]))

    out, all_dirs, dirs, rdict, log = outputs(run_dirs = self.run_dirs,
                                              start_date = self.start_date,
                                              end_date = self.end_date,
                                              filetype = self.filetype,
                                              wy = self.wy,
                                              loglevel = self.loglevel)

    # If there was an error parsing files catch and log it
    if out == [] and all_dirs == [] and 'not a valid file' in log[-1]:

        self.tmp_log.append(log[-1])
        if self.start_date is not None and self.end_date is not None:
            ext_shr = (self.directory +
                      '_'  +
                      self.start_date.date().strftime("%Y%m%d") +
                      '_' +
                      self.end_date.date().strftime("%Y%m%d") )
            self.figs_path = os.path.join(self.save_path, '{}/'.format(ext_shr))

            if external_logger == None:
                createLog(self)
            else:
                self._logger = external_logger
                
        raise Exception(log[-1])

    for l in log:
        self.tmp_log.append(l)

    if out['dates'] == []:
        raise Exception('Supplied [run] directory, start_date, and end_date '
              'give no valid snow files')

    self.outputs = out
    self.run_dirs = dirs
    self.all_dirs = all_dirs
    self.rundirs_dict = rdict
    self.all_dirs_flt = copy.deepcopy(all_dirs)

    if self.start_date is not None and self.end_date is None:
        self.end_date = self.outputs['dates'][-1]
        self.tmp_log.append(' Config options [run] end_date '
                            'not specified, assigning '
                            '{} and {}'.format(self.start_date,self.end_date))

        self.ixs = 0
        self.ixe = len(self.outputs['dates']) - 1

    # Otherwise, get closest dates and make indices
    else:
        self.start_date = self.outputs['dates'][0]
        self.end_date = self.outputs['dates'][-1]
        self.ixs = 0
        self.ixe = len(self.outputs['dates']) - 1

    if ((self.start_date.date() < self.outputs['dates'][0].date())
        or (self.end_date.date() > self.outputs['dates'][-1].date())):
        raise Exception('ERROR! Config option [run] start_date or end_date '
                        'outside of date range found in [run] directory')

    # Since model outputs at 23:00, step the figure and report dates to
    # show 00:00 the next day (unless start of water year)
    if self.start_date == datetime(self.wy-1,10,1,23,0,0):
        self.report_start = self.start_date

    else:
        self.report_start = self.start_date + timedelta(hours=1)

    # Copy the config file where figs will be saved
    # use directory if only plotting figures from database and don't
    # have start_date, end_date
    extf = os.path.splitext(os.path.split(self.config_file)[1])
    ext_shr = (self.directory +
              '_'  +
              self.start_date.date().strftime("%Y%m%d") +
              '_' +
              self.end_date.date().strftime("%Y%m%d") )
    self.figs_path = os.path.join(self.save_path, '{}/'.format(ext_shr))

    # get forecast outputs
    if self.forecast_flag:

        out, alldirs, dirs, rdict, log = outputs(run_dirs = self.for_run_dirs,
                                                 start_date = None,
                                                 end_date = None,
                                                 filetype = self.filetype,
                                                 wy = self.wy,
                                                 loglevel = self.loglevel)
        self.for_outputs = out
        self.for_run_dirs = dirs
        self.for_rundirs_dict = rdict
        self.for_ixs = 0
        self.for_ixe = len(self.for_outputs['swe_z']) - 1

    if self.flt_flag:

        file = self.update_file
        p = nc.Dataset(file, 'r')

        if self.update_numbers is None:
            times = p.variables['time'][:]
        else:
            if sum([x > len(p.variables['time']) for x in self.update_numbers]) > 0:
                self.tmp_log.append(' Value in [plots] update_numbers out of '
                                    'range, max is {}, flight update figs '
                                    'being set to False'.format(len(p.variables['time'])))
                times = []
                self.flt_flag = False

            else:
                times = p.variables['time'][self.update_numbers]

        p.close()

        flight_dates = []
        pre_flight_dates = []

        for time in times:
            wydate = calculate_date_from_wyhr(int(time), self.wy)
            pre_wydate = calculate_date_from_wyhr(int(time-24), self.wy)
            flight_dates = np.append(flight_dates,wydate)
            pre_flight_dates = np.append(pre_flight_dates,pre_wydate)

        if self.loglevel == 'DEBUG' and self.log_to_file is not True:
            print('Reading files in {} for flight updates...'.format(self.run_dirs[0].split('runs')[0]))

        out, x, dirs, rdict, log = outputs(run_dirs = self.all_dirs_flt,
                                           start_date = None,
                                           end_date = None,
                                           filetype = self.filetype,
                                           wy = self.wy,
                                           flight_dates = flight_dates,
                                           loglevel = self.loglevel)

        self.flight_outputs = out
        self.run_dirs_flt = dirs
        self.for_rundirs_dict = rdict
        self.flight_diff_dates = out['dates']

        out, x, dirs, rdict, log = outputs(run_dirs = self.all_dirs_flt,
                                           start_date = None,
                                           end_date = None,
                                           filetype = self.filetype,
                                           wy = self.wy,
                                           loglevel = self.loglevel,
                                           flight_dates = pre_flight_dates)

        self.pre_flight_outputs = out

        # If there are no flights in the period, set to false for the flight
        # difference figure and report
        if not self.run_dirs_flt:
            self.flt_flag = False
            self.tmp_log.append(' Config option [plots] update_file was '
                                'supplied, but no snow.nc files were found in '
                                '[run] directory that fit the date range, no '
                                'flight difference figure will be made')

    self.report_date = self.end_date + timedelta(hours=1)
    parts = self.report_name.split('.')
    self.report_name = (parts[0] + self.report_date.date().strftime("%Y%m%d") +
                       '.' + parts[1])

    if not os.path.exists(self.figs_path):
        os.makedirs(self.figs_path)

    config_copy = '{}{}{}'.format(self.figs_path, ext_shr, extf[1])
    generate_config(self.ucfg, config_copy)

    if external_logger == None:
        createLog(self)
    else:
        self._logger = external_logger

    if self.inputs_basins is None:
        self.inputs_basins = [self.plotorder[0]]

    # set up process() inputs for standard run
    self.pargs = {}
    self.pargs['outputs'] = self.outputs
    self.pargs['ixs'] = self.ixs
    self.pargs['ixe'] = self.ixe
    self.pargs['rundirs_dict'] = self.rundirs_dict
    self.pargs['edges'] = self.edges
    self.pargs['masks'] = self.masks
    self.pargs['nrows'] = self.nrows
    self.pargs['ncols'] = self.ncols
    self.pargs['plotorder'] = self.plotorder
    self.pargs['connector'] = self.connector
    self.pargs['basins'] = self.basins
    self.pargs['run_name'] = self.run_name
    self.pargs['db_overwrite'] = self.db_overwrite
    self.pargs['conversion_factor'] = self.conversion_factor
    self.pargs['depth_factor'] = self.depth_factor
    self.pargs['wy'] = self.wy
    self.pargs['vars'] = self.vars
    self.pargs['ixd'] = self.ixd
    self.pargs['vollbl'] = self.vollbl
    self.pargs['depthlbl'] = self.depthlbl
    self.pargs['elevlbl'] = self.elevlbl
    self.pargs['pixel'] = self.pixel
    self.pargs['dem'] = self.dem
    self.pargs['snow_limit'] = self.diag_limit
    self.pargs['inputs_flag'] = self.inputs_flag
    self.pargs['inputs_methods'] = self.inputs_methods
    self.pargs['inputs_basins'] = self.inputs_basins
    self.pargs['inputs_variables'] = self.inputs_variables
    self.pargs['inputs_percentiles'] = self.inputs_percentiles
    if self.mysql is not None:
        self.pargs['dbs'] = 'sql'
    else:
        self.pargs['dbs'] = 'sqlite'

def createLog(self):
    '''
    Create log file and print out saved logging statements.
    '''

    level_styles = {'info': {'color': 'white'},
                    'notice': {'color': 'magenta'},
                    'verbose': {'color': 'blue'},
                    'success': {'color': 'green', 'bold': True},
                    'spam': {'color': 'green', 'faint': True},
                    'critical': {'color': 'red', 'bold': True},
                    'error': {'color': 'red'},
                    'debug': {'color': 'green'},
                    'warning': {'color': 'yellow'}}

    field_styles =  {'hostname': {'color': 'magenta'},
                     'programname': {'color': 'cyan'},
                     'name': {'color': 'white'},
                     'levelname': {'color': 'white', 'bold': True},
                     'asctime': {'color': 'green'}}

    # start logging
    loglevel = self.loglevel
    numeric_level = getattr(logging, loglevel, None)

    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % loglevel)

    # setup the logging
    logfile = None
    if self.log_to_file:
        logfile = os.path.join(self.figs_path, 'log_snowav.out')
        # let user know
        print('Logging to file: {}'.format(logfile))

    fmt = '%(levelname)s:%(module)s:%(message)s'

    if logfile is not None:

        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)

        logging.basicConfig(filename=logfile,
                            filemode='w',
                            level=numeric_level,
                            format=fmt)
    else:
        logging.basicConfig(level=numeric_level)
        coloredlogs.install(level=numeric_level,
                            fmt=fmt,
                            level_styles=level_styles,
                            field_styles=field_styles)

    self._loglevel = numeric_level

    self._logger = logging.getLogger(__name__)

    if len(self.tmp_log) > 0:
        for l in self.tmp_log:
            self._logger.info(l)
    if len(self.tmp_warn) > 0:
        for l in self.tmp_warn:
            self._logger.warning(l)
    if len(self.tmp_err) > 0:
        for l in self.tmp_err:
            self._logger.error(l)




from snowav.database.database import collect
import pandas as pd

def write_properties(args, values):
    '''
    Write daily total snowpack properties to csv.

    Args
    ------
    args : dict
        dictionary with required inputs, see swi() figure for more information.
    values : str
        snowav database value to query and write to csv

    '''

    for value in values:
        out = collect(args['connector'], args['plotorder'], args['basins'],
                      args['wy_start'],args['end_date'],value,
                      args['run_name'],'total','daily')

        if 'vol' in value or 'avail' in value:
            unit = args['vollbl']

        if 'z' in value or value == 'depth':
            unit = args['depthlbl']

        if value == 'density':
            unit = 'kg_m3'

        if value == 'coldcont':
            unit = 'MJ'

        path = '{}{}_{}_{}.csv'.format(args['figs_path'], value,
                                       args['run_name'], unit)
        out.to_csv(path)




from datetime import datetime
import netCDF4 as nc
from snowav.plotting.swe_volume import swe_volume
from snowav.plotting.plotlims import plotlims as plotlims
from snowav.database.database import connect, make_session, collect
from snowav.utils.utilities import masks

"""
See README.md and CoreConfig.ini for more options and details.

"""

connector = 'sqlite:////<database.db>'
dempath = '<topo.nc>'
plotorder = ['Extended Tuolumne', 'Tuolumne', 'Cherry Creek', 'Eleanor']
lims = plotlims(plotorder)
edges = [3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000]

ncf = nc.Dataset('<snow.nc>')
image = ncf['specific_mass'][:]
ncf.close()

out = masks(dempath)
dem = out['dem']
masks = out['masks']
nrows = out['nrows']
ncols = out['ncols']
plotorder = out['plotorder']
labels = out['labels']
start_date = datetime(2019,6,1,23,0,0)
end_date = datetime(2019,6,2,23,0,0)
run_name = 'tuol_wy2019_devel'

barcolors = ['xkcd:cobalt', 'xkcd:mustard green', 'xkcd:lichen',
             'xkcd:pale green', 'xkcd:blue green']

basins, cnx, out = connect(sqlite=connector, mysql=None, plotorder=plotorder,
                           user=None, password=None, host=None, port=None)

args = {'report_start': '',
        'report_date': '',
        'print': False,
        'run_name': run_name,
        'start_date': start_date,
        'end_date': end_date,
        'directory': 'test',
        'figs_path': '<path>',
        'edges': edges,
        'plotorder': plotorder,
        'labels': labels,
        'lims': lims,
        'masks': masks,
        'figsize': (10,5),
        'dpi': 300,
        'depthlbl': 'in',
        'vollbl': 'TAF',
        'elevlbl': 'ft',
        'dplcs': 2,
        'barcolors': barcolors,
        'xlims': (0,len(edges)),
        'depth_clip': 0.01,
        'percent_min': 0.5,
        'percent_max': 99.5,
        'basins': basins,
        'title': 'SAMPLE'}

df = collect(connector, plotorder, basins, start_date, end_date, 'swe_vol',
             run_name, edges, 'end')

args['df'] = df
args['image'] = image[0,:,:]

swe_volume(args, logger = None)




import copy
import os
import numpy as np
from datetime import datetime, timedelta
from snowav.utils.wyhr import calculate_wyhr_from_date
from snowav.utils.OutputReader import iSnobalReader
import netCDF4 as nc

def outputs(run_dirs = None, start_date = None, end_date = None,
            filetype = None, wy = None, flight_dates = None, loglevel = None):
    '''
    This uses start_date and end_date to load the snow.nc and em.nc of interest
    within a report period to the outputs format that will be used in process().
    Also returns a clipped run_dirs that only contains paths with the specified
    date range. If start_date and end_date are not supplied, no run_dirs will
    be clipped.

    Note: this is assuming awsm_daily output folders and dates in the snow.nc
    file.

    Args
    -----
    run_dirs : list
        list of run directories
    start_date : datetime
        report period start date (optional)
    end_date : datetime
        report period end date (optional)
    filetype : str
        currently only supporting 'netcdf'
    wy : int
        water year
    flight_dates : array
        (optional)

    Returns
    ------
    outputs : dict
        dictionary of snow.nc and em.nc outputs within time period
    dirs : list
        all dirs within run_dir
    run_dirs : list
        modified run_dirs, with paths outside of start_date, end_date removed
    rdict : dict
        process() lookup

    '''

    log = []
    rdict = {}
    dirs = copy.deepcopy(run_dirs)
    outputs = {'swi_z':[], 'evap_z':[], 'snowmelt':[], 'swe_z':[],'depth':[],
               'dates':[], 'time':[], 'density':[], 'coldcont':[] }

    start = copy.deepcopy(start_date)
    end = copy.deepcopy(end_date)

    # Run this with standard processing, and forecast processing
    if flight_dates is None:

        for path in dirs:
            snowfile = os.path.join(path, 'snow.nc')

            if loglevel == 'DEBUG':
                log.append(' Reading date: {}'.format(snowfile))

            # Consider making this a warning, with an else: .remove(path)
            # to catch other files that are in these directories
            if not os.path.isfile(snowfile):
                log.append(' {} not a valid file'.format(snowfile))
                return [], [], [], [], log

            ncf = nc.Dataset(snowfile)
            ta = nc.num2date(ncf.variables['time'][:],ncf.variables['time'].units)
            ncf.close()

            ta = np.sort(ta)

            if start_date is None:
                start = copy.deepcopy(ta[0])

            if end_date is None:
                end = copy.deepcopy(ta[-1])

            for idx,t in enumerate(ta):

                # Only load the rundirs that we need
                if (t.date() >= start.date()) and (t.date() <= end.date()):

                    log.append(' Loading: {}'.format(snowfile))

                    st_hr = calculate_wyhr_from_date(start)
                    en_hr = calculate_wyhr_from_date(end)

                    output = iSnobalReader(path, filetype, snowbands = [0,1,2],
                                           embands = [6,7,8,9], wy = wy,
                                           time_start = st_hr, time_end = en_hr)

                    # Make a dict for wyhr-rundir lookup
                    for ot in output.time:
                        rdict[int(ot)] = path

                    outputs['swi_z'].append(output.em_data[8][idx,:,:])
                    outputs['snowmelt'].append(output.em_data[7][idx,:,:])
                    outputs['evap_z'].append(output.em_data[6][idx,:,:])
                    outputs['coldcont'].append(output.em_data[9][idx,:,:])
                    outputs['swe_z'].append(output.snow_data[2][idx,:,:])
                    outputs['depth'].append(output.snow_data[0][idx,:,:])
                    outputs['density'].append(output.snow_data[1][idx,:,:])
                    outputs['dates'].append(output.dates[idx])
                    outputs['time'].append(output.time[idx])

                else:
                    run_dirs.remove(path)

    # Run this when flight updates are present to make custom outputs
    else:
        for path in dirs:
            snowfile = os.path.join(path, 'snow.nc')

            # If the run_dirs isn't empty use it, otherwise remove
            if not os.path.isfile(snowfile):
                raise Exception('{} not a valid file'.format(snowfile))

            ncf = nc.Dataset(snowfile)
            ta = nc.num2date(ncf.variables['time'][:],ncf.variables['time'].units)
            ncf.close()

            for idx,t in enumerate(ta):
                if (t.date() in [x.date() for x in flight_dates]):

                    output = iSnobalReader(path, filetype, snowbands=[0,1,2], wy=wy)

                    for ot in output.time:
                        rdict[int(ot)] = path

                    outputs['swe_z'].append(output.snow_data[2][idx,:,:])
                    outputs['depth'].append(output.snow_data[0][idx,:,:])
                    outputs['density'].append(output.snow_data[1][idx,:,:])
                    outputs['dates'].append(output.dates[idx])
                    outputs['time'].append(output.time[idx])

    return outputs, dirs, run_dirs, rdict, log




import os
from datetime import datetime
from sqlalchemy import Column, ForeignKey, Integer, String, create_engine, and_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import backref, sessionmaker
from snowav.database.tables import Base, RunMetadata, Watershed, Basin, Results, VariableUnits
import pandas as pd
import snowav
import numpy as np
import copy
import urllib.parse
import mysql.connector
from sys import exit
import warnings

# Fix these two by pulling smrf and awsm versions from netcdf
try:
    import smrf
    smrf_version = smrf.__version__
except:
    print('Could not import smrf, database smrf version will be "unknown"')
    smrf_version = 'unknown'

try:
    warnings.filterwarnings("ignore")
    import awsm
    warnings.filterwarnings("default")
    awsm_version = awsm.__version__

except:
    print('Could not import awsm, database smrf version will be "unknown"')
    awsm_version = 'unknown'

def make_session(connector):
    '''
    Make snowav database session.

    Args
    -------
    connector : str
        database connector, either path to sqlite database or mysqlconnector
        string

    Returns
    -------
    session : object
        snowav database session
    '''

    try:
        engine = create_engine(connector)
        Base.metadata.create_all(engine)

    except:
        raise Exception('Failed to make database connection with '
                        '{}'.format(connector))

    DBSession = sessionmaker(bind=engine)
    session = DBSession()

    return session

def insert(connector, table, values):
    '''
    Inserts standard run results to the database.

    Args
    --------
    session : object
        database session
    table : str
        database table (RunMetadata, Results, or VariableUnits)
    values : dict
        dictionary of values

    '''

    dbtable = getattr(snowav.database.tables, table)
    my_dbtable = dbtable()

    for k,v in values.items():
        setattr(my_dbtable, k, v)

    session = make_session(connector)
    session.add(my_dbtable)
    session.commit()
    session.close()

def collect(connector, plotorder, basins, start_date, end_date, value,
            run_name, edges, method):
    '''

    Collect snowav database results into our standard dataframe with elevation
    rows and subbasin columns.

    Args
    ---------
    session : object
        sqlalchemy snowav database session
    plotorder : str
        list of basins
    start_date : datetime
    end_date : datetime
    value : str
        database value identifier
    run_name : str
        database run name identifier
    edges : list
        elevation band, can include 'total'
    method : str
        options [end sum difference daily]

    Returns
    ----------
    df : DataFrame
        standard subbasin columns and elevation labels

               Extended Tuolumne  Tuolumne  Cherry Creek  Eleanor
        3000               0.000     0.000         0.000    0.000
        ...                  ...       ...           ...      ...

    '''

    value_options = ['swe_z','swe_vol','density','precip_z','precip_vol',
                     'rain_z','rain_vol','swe_avail','swe_unavail','coldcont',
                     'swi_z','swi_vol','depth','snow_line','mean_air_temp']

    method_options = ['sum','difference','daily','end']

    if value not in value_options:
        raise Exception('value={} call to database.collect() '.format(value) +
                        'must be one of {}'.format(value_options))

    if method not in method_options:
        raise Exception('method={} call to database collect() '.format(method) +
                        'must be one of {}'.format(method_options))

    if type(plotorder) != list:
        plotorder = [plotorder]

    if edges == 'total':
        edges = ['total']

    if method == 'daily':
        df = pd.DataFrame(index = ['date_time'], columns=plotorder)

    else:
        df = pd.DataFrame(np.nan, index=edges, columns=plotorder)

    for bid in plotorder:
        results = query(connector, start_date, end_date, run_name, basins, bid, value)

        if results.empty:
            raise Exception('Empty results for database query in '
                            'database.collect(), check config file '
                            'start_date, end_date, and that the run_name={} '
                            'has results for {} to {}'. format(run_name,
                            start_date, end_date))

        if method == 'daily':
            e = results[(results['elevation'] == 'total')
                        & (results['date_time'] >= start_date)
                        & (results['date_time'] <= end_date)]
            if e.empty:
                raise Exception('Empty results for database query in '
                                'database.collect(), for run_name={}, '
                                'elev={}, {} to {}'. format(run_name, elev,
                                start_date, end_date))
            else:
                e = e.set_index('date_time')
                e.sort_index(inplace=True)

                if bid == plotorder[0]:
                    df = e[['value']].copy()
                    df = df.rename(columns={'value':bid})

                else:
                    df[bid] = e['value']

            df.sort_index(inplace=True)

        if method == 'end':
            for elev in edges:
                e = results[(results['elevation'] == str(elev))
                            & (results['date_time'] == end_date)]
                if e.empty:
                    raise Exception('Empty results for database query in '
                                    'database.collect(), for run_name={}, '
                                    'elev={}, {} to {}'. format(run_name, elev,
                                    start_date, end_date))
                else:
                    if e['value'].values[0] is None:
                        df.loc[elev,bid] = np.nan
                    else:
                        df.loc[elev,bid] = e['value'].values


        if method == 'difference':
            for elev in edges:
                s = results[(results['elevation'] == str(elev))
                            & (results['date_time'] == start_date)]
                e = results[(results['elevation'] == str(elev))
                            & (results['date_time'] == end_date)]
                if e.empty or s.empty:
                    raise Exception('Empty results for database query in '
                                  'database.collect(), for run_name={}, '
                                  'elev={}, {} to {}'. format(run_name, elev,
                                  start_date, end_date))
                else:
                    df.loc[elev,bid] = np.nansum(e['value'].values-s['value'].values)

        if method == 'sum':
            for elev in edges:
                e = results[(results['elevation'] == str(elev))
                            & (results['date_time'] >= start_date)
                            & (results['date_time'] <= end_date)]
                if e.empty:
                    raise Exception('Empty results for database query in '
                                    'database.collect(), for run_name={}, '
                                    'elev={}, {} to {}'. format(run_name, elev,
                                    start_date, end_date))
                else:
                    df.loc[elev,bid] = e['value'].sum(skipna=False)

    df = df[plotorder]

    return df

def query(connector, start_date, end_date, run_name, basins, bid = None,
          value = None, rid = None):
    '''
    Retrieve results from snowav database.

    Args
    --------
    start_date : datetime
    end_date : datetime
    run_name : str
        identifier for run, specified in config file
    bid : str
        basin id in string format ('Boise River Basin')
    value : str
        value to query (i.e. 'swi_z')
    rid : run_id

    Returns
    ---------
    df : DataFrame
        dataframe of query results

    '''

    basin_id = int(basins[bid]['basin_id'])
    session = make_session(connector)

    if (value != None) and (bid != None):
        qry = session.query(Results).join(RunMetadata).filter(and_(
                        (Results.date_time >= start_date),
                        (Results.date_time <= end_date),
                        (RunMetadata.run_name == run_name),
                        (Results.variable == value),
                        (Results.basin_id == basin_id)))

    elif (value == None) and (bid != None) and rid != None:
        qry = session.query(Results).join(RunMetadata).filter(and_(
                                            (Results.date_time >= start_date),
                                            (Results.date_time <= end_date),
                                            (Results.run_id == int(rid)),
                                            (RunMetadata.run_name == run_name),
                                            (Results.basin_id.in_(bids))))

    else:
        qry = session.query(Results).join(RunMetadata).filter(and_(
                                            (Results.date_time >= start_date),
                                            (Results.date_time <= end_date),
                                            (RunMetadata.run_name == run_name)))

    df = pd.read_sql(qry.statement, qry.session.connection())

    session.close()

    return df

def delete(connector, basins, start_date, end_date, bid, run_name):
    '''
    Delete results from the database. This deletes values with basin_id == bid
    and run_name = run_name within the date range. If there are no more
    remaining records for that run_id (the entire run was deleted), then the
    RunMetadata and VariableUnits data are also removed.

    Note: deletion order matters because of table relationships.

    Args
    --------
    start_date : datetime
    end_date : datetime
    bid : str
        basin name
    run_name : str
        identifier for run, specified in config file

    Returns
    --------
    logger : list

    '''

    logger = []
    basin_id = int(basins[bid]['basin_id'])
    session = make_session(connector)

    logger.append(' Deleting existing records for {}, {}, {} '.format(
                  bid, run_name, start_date.date()))

    # Get the run_id
    qry = session.query(Results).join(RunMetadata).filter(and_(
                                        (Results.date_time >= start_date),
                                        (Results.date_time <= end_date),
                                        (RunMetadata.run_name == run_name),
                                        (Results.basin_id == basin_id)))

    df = pd.read_sql(qry.statement, qry.session.connection())
    unique_runs = df.run_id.unique()

    if not df.empty:
        for r in unique_runs:
            session.query(Results).filter(and_((Results.date_time >= start_date),
                                    (Results.date_time <= end_date),
                                    (Results.run_id == int(r)),
                                    (Results.basin_id == basin_id))).delete()

            session.commit()

        # Query again, if no results from those run_id exist still, also remove
        # the metadata and variableUnits
        qrym = session.query(Results).filter(Results.run_id == int(r))
        dfm = pd.read_sql(qrym.statement, qrym.session.connection())

        if dfm.empty:
            logger.append(' Deleting RunMetadata run_name={}, run_id={}, from {} '
                          'to {}'.format(run_name, str(r), start_date.date(),
                          end_date.date()))

            session.query(VariableUnits).filter(VariableUnits.run_id == int(r)).delete()
            session.query(RunMetadata).filter(RunMetadata.run_id == int(r)).delete()
            session.commit()

    session.close()

    return logger

def create_tables(database, plotorder):
    '''
    Creates database Watersheds and Basins tables. This is currently only
    being called on database creation.

    Args
    -------
    database : str
        database identifier
    plotorder : list

    Returns
    --------
    logger : list


    '''

    logger = []

    engine = create_engine(database)
    Base.metadata.create_all(engine)
    DBSession = sessionmaker(bind=engine)
    session = DBSession()

    logger.append(' Adding watershed {} to database'.format(plotorder[0]))

    # Initialize watershed
    wval = Watershed(watershed_id = 1, watershed_name = plotorder[0])
    session.add(wval)
    session.commit()

    # Initialize basins within the watershed
    for i,name in enumerate(plotorder):
        logger.append(' Adding basin {} to database'.format(name))

        bval = Basin(watershed_id = 1,
                     basin_id = i + 1,
                     basin_name = name)

        session.add(bval)

    session.commit()
    session.close()

    logger.append(' Completed initializing basins and watersheds')

    return logger

def basins_dict():
    '''
    Make the basins lookup dictionary.

    '''

def run_metadata(self, run_name):
    '''
    Create database RunMetadata for each snowav run.

    Args
    ------
    run_name : str

    '''

    watershed_id = int(self.basins[self.plotorder[0]]['watershed_id'])

    session = make_session(self.connector)
    qry = session.query(RunMetadata)
    df = pd.read_sql(qry.statement, qry.session.connection())
    session.close()

    # Increase each runid by 1
    if not df.run_id.values.size:
        self.run_id = 1

    else:
        self.run_id = int(df['run_id'].max() + 1)

    # in most cases it's not practical to save all run_dirs
    trdir = ','.join(self.run_dirs)
    if len(trdir) > 1200:
        trdir = self.run_dirs[0]

    values = {'run_id':int(self.run_id),
              'run_name':run_name,
              'watershed_id':int(watershed_id),
              'pixel':int(self.pixel),
              'description':'',
              'smrf_version':'smrf'+ smrf_version,
              'awsm_version':'aswm'+ awsm_version,
              'snowav_version':'snowav'+ snowav.__version__,
              'data_type':'',
              'data_location':trdir,
              'file_type':'',
              'config_file':self.config_file,
              'proc_time':datetime.now() }

    insert(self.connector, 'RunMetadata', values)

    self.vid = {}
    for v in self.vars.keys():
        if (('vol' in v) or ('avail' in v)):
            u = self.units
        if (('z' in v) or (v == 'depth')):
            u = self.depthlbl
        if v == 'density':
            u = 'kg m^-3'
        if v == 'coldcont':
            u = 'MJ'

        variables = {'run_id':self.run_id,
                     'variable':v,
                     'unit':u,
                     'name':self.vars[v]}

        insert(self.connector,'VariableUnits',variables)

        # After inserting into VariableUnits, get the existing id
        session = make_session(self.connector)
        qry = session.query(VariableUnits).filter(VariableUnits.run_id == self.run_id)
        session.close()
        df = pd.read_sql(qry.statement, qry.session.connection())
        self.vid[v] = df[df['variable'] == v]['id'].values[0]

    # snow_line
    variables = {'run_id':self.run_id,
                 'variable':'snow_line',
                 'unit':self.elevlbl,
                 'name':'snow_line'}

    insert(self.connector,'VariableUnits',variables)
    session = make_session(self.connector)
    qry = session.query(VariableUnits).filter(VariableUnits.run_id == self.run_id)
    session.close()
    df = pd.read_sql(qry.statement, qry.session.connection())
    self.vid['snow_line'] = df['id'].values[0]

    # Add post-processing values
    sum_vals = {'swi_vol_wy':'surface water input volume, water year total',
                'swi_z_wy':'surface water input depth, water year total',
                'evap_z_wy':'evaporation depth, water year total'}

    for v in sum_vals.keys():
        if ('vol' in v):
            u = self.units
        if ('z' in v) :
            u = self.depthlbl

        variables = {'run_id':self.run_id,
                     'variable':v,
                     'unit':u,
                     'name':sum_vals[v]}

        insert(self.connector,'VariableUnits',variables)

        session = make_session(self.connector)
        qry = session.query(VariableUnits).filter(VariableUnits.run_id == self.run_id)
        session.close()
        df = pd.read_sql(qry.statement, qry.session.connection())

        self.vid[v] = df[df['variable'] == v]['id'].values[0]


def connect(sqlite = None, sql = None, plotorder = None, user = None,
            password = None, host = None, port = None, convert = False,
            add = False):
    '''
    This establishes a connection with a database for results. If the specified
    sqlite database doesn't exist, it will be created.

    Note: consider moving creation of mysql database to Makefile

    Args
    --------
    sqlite : str
        path to sqlite database
    sql : str
        snowav mysql identifier
    plotorder : list
        list of basins
    user : str
        database user
    password : str
        database password
    host : int
        database host
    port : int
        database port

    Returns
    ---------
    basins : dict
        dictionary of snowav basins and ids for database connection
    logger : list

    '''

    logger = []

    if sqlite is not None:
        fp = urllib.parse.urlparse(sqlite)

        if (not os.path.isfile(fp.path)):
            database = sqlite
            logger.append(' Creating {} for results'.format(database))
            log = create_tables(database, plotorder)

            for out in log:
                logger.append(out)

            engine = create_engine(sqlite)

        else:
            engine = create_engine(sqlite)

        DBSession = sessionmaker(bind=engine)
        session = DBSession()
        logger.append(' Using {} for results'.format(sqlite))
        connector = sqlite

    if (sql is not None) and (sqlite is None):

        try:
            cnx = mysql.connector.connect(user=user,
                                          password=password,
                                          host=host,
                                          port=port)
        except:
            print('Failed attempting to make database connection with '
                  '{}:{}@{}/{}\nCheck config options in [database] '
                  'section'.format(user,password,host,port))
            exit()
        cursor = cnx.cursor()

        # Check if database exists, create if necessary
        query = ("SHOW DATABASES")
        cursor.execute(query)
        dbs = cursor.fetchall()

        try:
            dbs = [i[0].decode("utf-8") for i in dbs]
        except:
            dbs = [i[0] for i in dbs]

        db_engine = 'mysql+mysqlconnector://{}:{}@{}/{}'.format(user,
                                                                password,
                                                                host,
                                                                sql)

        # If the database doesn't exist, create it, otherwise connect
        if (sql not in dbs):
            logger.append(' Specified mysql database {} '.format(sql) +
                  ' does not exist, it is being created...')
            query = ("CREATE DATABASE {};".format(sql))
            cursor.execute(query)
            log = create_tables(db_engine, plotorder)

            for out in log:
                logger.append(out)

            logger.append(log)

        else:
            query = ("SHOW DATABASES")
            cursor.execute('USE {}'.format(sql))
            cursor.execute('SHOW TABLES')
            tbls = cursor.fetchall()

            # This hasn't been tested with new docker mysql instance, could be
            # a point of failure...
            if tbls is None:
                log = create_tables(db_engine, plotorder)

                for out in log:
                    logger.append(out)

                logger.append(log)

            try:
                logger.append(' Using database connection {}@{}/{} for '
                              'results'.format(user, host, sql))
                engine = create_engine(db_engine)
                Base.metadata.create_all(engine)
                DBSession = sessionmaker(bind=engine)
                session = DBSession()

            except:
                logger.append(' Failed trying to make database connection '
                      'to {}'.format(sql))

        cursor.close()
        cnx.close()

        connector = db_engine

    qry = session.query(Watershed)
    ws = pd.read_sql(qry.statement, qry.session.connection())

    qry = session.query(Basin)
    bs = pd.read_sql(qry.statement, qry.session.connection())

    if convert:
        plotorder[0] = convert_watershed_names(plotorder[0])

    if plotorder[0] in ws['watershed_name'].values:
        f = ws[ws['watershed_name'].values == plotorder[0]]
        wid = f['watershed_id'].values[0]

    else:
        logger.append(' {} not found as a watershed in '.format(plotorder[0]) +
                      'existing database, it is being added')

        if not ws.empty:
            wid = np.max(ws['watershed_id'].values) + 1
        else:
            wid = 1

        wval = Watershed(watershed_id = int(wid),
                         watershed_name = plotorder[0])
        if add:
            session.add(wval)

        else:
            logger.append(' Config option [database] add_basins: False, so '
                          '{} will not be added'.format(plotorder[0]))

            print('WARNING! Given current config options and topo.nc file, '
                  'trying to use {} as a watershed, but it does not currently '
                  'exist as a watershed in the database...\nCorrect [snowav] '
                  'masks or consider changing [database] add_basins: True '
                  'if you know what you are doing.'.format(plotorder[0]))

        session.commit()

    basins = {}
    watersheds = {plotorder[0]:{'watershed_id':wid, 'watershed_name':'',
                                'basins': '', 'shapefile':''}}

    # Initialize basins within the watershed
    for i,name in enumerate(plotorder):
        if name in bs['basin_name'].values:
            bid = int(bs[(bs['watershed_id'] == wid) & (bs['basin_name'] == name)]['basin_id'].values)

        else:
            logger.append(' {} not found as a basin in '.format(name) +
                          'existing database, it is being added')

            if not bs.empty:
                bid = np.max(bs['basin_id'].values) + 1
            else:
                bid = 1

            bval = Basin(watershed_id = int(wid),
                         basin_id = int(bid + i),
                         basin_name = name)

            if add:
                session.add(bval)

            else:
                logger.append(' Config option [database] add_basins: False, so '
                              '{} will not be added'.format(name))

                print('WARNING! Given current config options and topo.nc file, '
                      'trying to use {} as a basin, but it does not currently '
                      'exist as a basin in the database...\nConsider '
                      'changing [database] add_basins: True if you know what '
                      'you are doing.'.format(name))

        basins[name] = {'watershed_id':wid, 'basin_id':bid}

    session.commit()
    session.close()

    return basins, connector, logger

def convert_watershed_names(name):
    '''
    Convert watershed from topo.nc mask to existing snowav database version.

    '''
    convert_list = ['Boise River Basin','Lakes Basin','Merced River Basin',
                    'Kaweah River Basin','Kings River Basin']

    if name in convert_list:
        watershed = name.split(' ')[0]

    elif name == 'San Joaquin River Basin':
        watershed = 'San Joaquin'

    else:
        watershed = name

    return watershed




from snowav.utils.utilities import calculate
import netCDF4 as nc

"""
Sample processing script for snowav processing utility. Calculate values from
iSnobal output snow.nc and em.nc files and convert to desired units, including:
    mean SWE depth
    total SWE volume
    mean snow depth
    total SWI volume
    mean SWI depth
    mean evaporation
    total cold content
    mean density

Applies all masks supplied, pass multiple masks as a list.

"""

# Specify the iSnobal output file and topo.nc for the basin mask (optional).
file_path = '/data/blizzard/tuolumne/ops/wy2019/ops/runs/run20190101/snow.nc'
topo_path = '/home/ops/wy2019/tuolumne/topo/topo.nc'
value = 'specific_mass'
pixel = 50

# Get desired output value
ncf = nc.Dataset(file_path)
array = ncf.variables[value][:]
ncf.close()

# Optional - get total basin mask from the topo.nc file.
# Can also pass additional masks, grouped in a list
ncf = nc.Dataset(topo_path)
mask = ncf['mask'][:]
ncf.close()

# SWE volume [TAF]
swe_vol = calculate(array[0], pixel, mask, 'sum', 'volume', 'TAF')

# mean SWE depth [in]
swe_in = calculate(array[0], pixel, mask, 'mean', 'depth', 'TAF')
